//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

const FLOCK_BLOG_TOPBAR_POSTID_FLAVOUR = "flock_blog_post_id";

var gSelectedPost = null;
var gBlogCommandController = null;

function flock_blogBuildPostURI(postid) {
    var url = "chrome://browser/content/flock/blog/tinyBlog.xul"
    var params = "?postid=" + hexEncode(postid);
    url += params;
    return url;
}

function flock_blogTopbarUnload() {
    flock_blogRemoveEditorCommands();
    var blogSettings = flock_blogGetSettings();
    var ds = blogSettings.ds;
    ds.getRawDataSource().RemoveObserver(gBlogTopbarRDFObserver);
}

function flock_blogTopbarInitUI() {

    var blogSettings = flock_blogGetSettings();
    var ds = blogSettings.ds;

    var postbox = $("flock-blog-topbar-posts");
    postbox.database.AddDataSource(ds.getRawDataSource());
    postbox.ref = FLOCK_BLOG_RDF_ALLPOSTS_FOLDER;
    postbox.builder.rebuild();

    flock_blogTopbarUpdateDeck();

    ds.getRawDataSource().RemoveObserver(gBlogTopbarRDFObserver);
    ds.getRawDataSource().AddObserver(gBlogTopbarRDFObserver);
    $("blog-account-input").select();
}

function flock_blogTopbarLoad() {
    try {
        var blogSettings = flock_blogGetSettings();
        blogSettings.rebuildAllPosts();
        flock_blogSetupTopbarCommands();
        flock_blogTopbarInitUI();

        window.addEventListener('focus',function () { window.document.commandDispatcher.updateCommands('focus'); },false);
        window.addEventListener('select',function () { window.document.commandDispatcher.updateCommands('select'); },false);
        window.focus();
        window.document.commandDispatcher.updateCommands('focus');
    }
    catch(e) {
        alert(e + e.fileName + e.lineNumber);
    }
}

function flock_blogDownloadPostsIfNeedBe() {
    var blogSettings = flock_blogGetSettings();
    if(blogSettings.needRefresh()) {
        flock_blogRefreshAllAccountsCommand.exec();
    }
}

function flock_blogTopbarUpdateDeck() {
    var blogSettings = flock_blogGetSettings();
    var accountCount = blogSettings.getAccountCount();
    if (accountCount<2) {
        $("flock_blogTopbarDeck").selectedIndex = 0;
    }
    else {
        $("flock_blogTopbarDeck").selectedIndex = 1;
    }

    flock_blogTopbarRefresh();
//    $("flock-blog-topbar-posts").builder.rebuild();
    flock_blogDownloadPostsIfNeedBe();
}

function flock_blogTopbarSelectAccount(aEvent) {
    var blog_title = aEvent.target.label;

    var blog = BLOG_URN_ROOT + blog_title;
}

function flock_blogTopbarSelectPost(postid) {
    window.document.commandDispatcher.updateCommands('focus');
    gSelectedPost = postid;
}

var flock_blogTopbarDDObserver =
{
    onDragOver: function(aEvent,aFlavor,aSession) {
    },

    onDrop: function(aEvent,aDropData,aSession) {
        //topWindow.flock_blogBlogThisCommand.target = aSession.sourceNode;
//        alert("drop");
        XXX("dropaasdasdasdasdasdasdsadasdsaddsadaasdasdasdasdasdasdsadasdsaddsadaasdasdasdasdasdasdsadasdsaddsadaasdasdasdasdasdasdsadasdsaddsadssss");
        var topWindow = flock_mostRecentNavigator();
        topWindow.flock_blogBlogThisCommand.onDrop(aSession.sourceNode);
//        alert(topWindow.flock_blogBlogThisCommand.target + "jj");
//        alert(aSession.sourceNode);
//        alert(aEvent.target.nodeName);
//        flock_goDoCommand("cmd_flock_blogBlogThis");
    },

    onDragStart: function(aEvent, aTransferData, aAction) {
        return;
        //var postid = aEvent.target.id;
        var postid = "dummy";
        aTransferData.data = new TransferData();
        aTransferData.data.addDataForFlavour(FLOCK_BLOG_TOPBAR_POSTID_FLAVOUR,postid);
    },

    getSupportedFlavours: function() {
        var flavors = new FlavourSet();
        flavors.appendFlavour("text/html");
			return flavors;
    }
}

function flock_blogTopbarRefresh() {
    $("flock_blogTopbarDeck").setAttribute("collapsed", true);
    window.setTimeout('$("flock_blogTopbarDeck").removeAttribute("collapsed");', 10);
}


/************************************************************************/
// RDF Observer
/************************************************************************/


var gBlogTopbarRDFObserver = {
    lastUpdate: 0,
    update: function() {
        var now = new Date();
        now = now.getTime();
        if(now - this.lastUpdate>1000) {
            window.setTimeout("flock_blogTopbarUpdateDeck()",500);
        }
        this.lastUpdate = now;
    },
    onAssert: function(ds, source, predicate, target) {
        if(predicate.Value == (FLOCK_BLOG_NS + "postid")) {
        }
        if(predicate.Value == (FLOCK_BLOG_NS + "posts")) {
            this.update();
        }
    },
    onUnassert: function(ds, source, predicate, target) {
        if(predicate.Value == (FLOCK_BLOG_NS + "postid")) {
        }
        if(predicate.Value == (FLOCK_BLOG_NS + "posts")) {
            this.update();
        }
    },
    onChange: function(ds, source, predicate, oldTarget, target) {
    },
    onMove: function(ds, oldSource, newSource, predicate, target) {
    },
}

/////////////////////////////////////////////////////////////////
// Commands for the UI
/////////////////////////////////////////////////////////////////


var flock_blogTopbarEditPostCommand = {
    isEnabled: function() {
        return gSelectedPost != null;
    },
    exec: function() {
        var url = flock_blogBuildPostURI(gSelectedPost);
        flock_blogDoOpenPost(url);
    },
}

var flock_blogTopbarGotoBlogCommand = {
    update: function() {
        this.permaLink = null;
        if(!gSelectedPost) return;
        gSelectedPost.match(/(.+):posts:(.*)/);
        var postid = RegExp.$2;
        var account = RegExp.$1;
        if(account==FLOCK_BLOG_DRAFTS_FOLDER) return false;
        var blogSettings = flock_blogGetSettings();
        var blogAccount = blogSettings.getAccount(account);
        this.permaLink = blogAccount.getPostByURI(gSelectedPost).permaLink;
    },
    isEnabled: function() {
        this.update();
        if(!this.permaLink) return false;
        return true;
        /*
        if(!this.permaLink) return false;
        if(!gSelectedPost) return false;
        gSelectedPost.match(/(.+):posts:(.*)/);
        var postid = RegExp.$2;
        var account = RegExp.$1;
        if(account==FLOCK_BLOG_DRAFTS_FOLDER) return false;
        var blogSettings = flock_blogGetSettings();
        var blogAccount = blogSettings.getAccount(account);
        var permaLink = blogAccount.getPostByURI(gSelectedPost).permaLink;
        //alert(permaLink);
        if(!blogAccount.url) return false;
        */
    },
    exec: function() {
//        alert(this.isEnabled);
        flock_blogTopbarGotoBlogCommand.update();
        gSelectedPost.match(/(.+):posts:(.*)/);
        flock.browser.open(flock_blogTopbarGotoBlogCommand.permaLink, "foreground_tab");
/*
        var postid = RegExp.$2;
        var account = RegExp.$1;
        var blogSettings = flock_blogGetSettings();
        var blogAccount = blogSettings.getAccount(account);
        flock_goto(blogAccount.url);
*/
        //var url = flock_blogBuildPostURI(gSelectedPost);
        //flock_blogDoOpenPost(url);
    },
}

var flock_blogTopbarDeletePostCommand = {
    isEnabled: function() {
        return gSelectedPost != null;
    },
    exec: function() {
        gSelectedPost.match(/(.+):posts:(.*)/);
        var postid = RegExp.$2;
        var account = RegExp.$1;

        var deleteListener = {
            onResult: function(result) { 
            },

            onFault: function (fault) { 
                alert("Oops - there was an error deleting this post from your service\n You may have to log onto your service and delete this post manually.");
            },

            onError: function (errorMsg) { 
                alert("Oops - there was an error deleting this post from your service\n You may have to log onto your service and delete this post manually.");
            },
        };


        var blogSettings = flock_blogGetSettings();
        var blogAccount = blogSettings.getAccount(account);
        var blogPost = blogAccount.getPostByURI(gSelectedPost);
        if(account!=FLOCK_BLOG_DRAFTS_FOLDER) {
            var B = blogAccount.getBlogAPI();
            B.deletePost(deleteListener, postid, blogPost.editURI);
        }

        blogAccount.deletePost(blogPost);
        gSelectedPost = null;

    },
}

var flock_blogTopbarCreateAccountCommand = {
    isEnabled: function() {
        return true;
    },
    exec: function() {
        window.openDialog("blogAccountWizard.xul", "accountWizard", "modal", ""); 
        //window.setTimeout("flock_blogTopbarUpdateDeck()",100);
    },
}

function flock_blogRemoveEditorCommands() {
    try {
        window.controllers.removeController(gBlogCommandController);
    }
    catch(e) {
    }
}

function flock_blogSetupTopbarCommands() {
    if(gBlogCommandController) return;
    gBlogCommandController = new CommandController();

    gBlogCommandController.cmd_flock_blogGotoBlog = flock_blogTopbarGotoBlogCommand;
    gBlogCommandController.cmd_flock_blogEditPost = flock_blogTopbarEditPostCommand;
    gBlogCommandController.cmd_flock_blogDeletePost = flock_blogTopbarDeletePostCommand;
    gBlogCommandController.cmd_flock_blogCreateAccount = flock_blogTopbarCreateAccountCommand;
    gBlogCommandController.cmd_flock_blogRefreshAllAccounts = flock_blogRefreshAllAccountsCommand;
    gBlogCommandController.cmd_flock_blogBlogThis = flock_blogBlogThisCommand;
    gBlogCommandController.cmd_flock_blogNewPost = flock_blogNewPostCommand;

    window.controllers.appendController(gBlogCommandController);
    window.focus();
}

