// vim: ts=4 sw=4 cindent expandtab 

const SOCKETTRANSPORT_CONTRACTID = 
        '@mozilla.org/network/socket-transport-service;1';
const SCRIPTABLEINPUTSTREAM_CONTRACTID = 
        '@mozilla.org/scriptableinputstream;1';
const INPUTSTREAMPUMP_CONTRACTID = 
        '@mozilla.org/network/input-stream-pump;1';

const nsISocketTransportService = 
        Components.interfaces.nsISocketTransportService;
const nsIScriptableInputStream = 
        Components.interfaces.nsIScriptableInputStream;
const nsIInputStreamPump = 
        Components.interfaces.nsIInputStreamPump;

var httpClient = {};
httpClient.log = function (msg) { }
httpClient.base64 = function (aInput) {
    var chars = 
        'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';
    var output = '';
    while (aInput.length > 0) {
        output += chars[aInput.charCodeAt(0) >> 2];
        output += chars[((aInput.charCodeAt(0)&0x03) << 4) |
            (aInput.length>1?((aInput.charCodeAt(1)&0xF0) >> 4):0)];
        output += chars[aInput.length>1?
            ((aInput.charCodeAt(1)&0x0F)<<2) |
            (aInput.length>2?((aInput.charCodeAt(2)&0xC0) >> 6):0):64];
        output += chars[aInput.length>2?
            (aInput.charCodeAt(2)&0x3F):64];
        if (aInput.length > 3) {
            aInput = aInput.substr (3);
        } else {
            break;
        }
    }
    return output;
}
httpClient.GET = function (host, port, path, extra_headers, listener) {
    try {
        var transportService =
            Components.classes[SOCKETTRANSPORT_CONTRACTID]
            .getService (nsISocketTransportService);
        var transport = transportService.createTransport (null, 0, host, port, 
                null);

        var outputData = 'GET '+path+' HTTP/1.1\r\n';
        outputData += 'Host: '+host+'\r\n';
        outputData += 'Connection: close\r\n';
        for (var name in extra_headers) {
            outputData  += name  +  ': ' + extra_headers[name] + '\r\n';
        }
        outputData += '\r\n';

        httpClient.log ('connecting to '+host+':'+port+'\n');
        httpClient.log ('sending:['+outputData+']\n');

        var outstream = transport.openOutputStream (0, 0, 0);
        outstream.write (outputData, outputData.length);

        var stream = transport.openInputStream (0, 0, 0);
        var instream = Components.classes[SCRIPTABLEINPUTSTREAM_CONTRACTID]
            .createInstance (nsIScriptableInputStream);
        instream.init (stream);

        var dataListener = {
            data : '', 
            onStartRequest: function (request, context){
                httpClient.log ('onStartRequest\n');
            }, 
            onStopRequest: function (request, context, status){
                httpClient.log ('onStopRequest status='+status+'\n');
                instream.close ();
                outstream.close ();

                if (status != 0) {
                    listener (status)
                    return
                }

                var parts = this.data.split ('\r\n\r\n');
                var hdrs = parts[0].split ('\r\n');
                var body = parts[1];

                parts = hdrs[0].match (/HTTP\/1.[01] *(\d\d\d) (.*)/);
                status = parseInt (parts[1]);
                var message = parts[2];

                var headers = {};
                for (var i=1; i<hdrs.length; i++) {
                    parts = hdrs[i].split ('/: */', 1)
                    headers[parts[0]] = parts[1];
                }

                listener (status, message, headers, body);
                httpClient.log ('onStopRequest done\n');
            }, 
            onDataAvailable: function (request, context, inputStream, offset, count) {
                httpClient.log ('onDataAvailable\n');
                this.data += instream.read (count);
            }, 
        };

        var pump = Components.classes[INPUTSTREAMPUMP_CONTRACTID] 
            .createInstance (nsIInputStreamPump);
        pump.init (stream, -1, -1, 0, 0, false);
        pump.asyncRead (dataListener, null);
    } catch (ex){
        httpClient.log ('exception: '+ex+'\n');
        return ex;
    }

    return null;
}
