// vim: foldmethod=marker
flock.require("flock.favorites.favorites");
flock.require("flock.common.rdf");
flock.require("flock.common.xpcom");

// Migration
flock.favorites.bookmarksMigrator = function (aOpts) {
    flock.rdf.RDFSniffer.call(this, aOpts);
}
    
dojo.inherits(flock.favorites.bookmarksMigrator, flock.rdf.RDFSniffer);
dojo.lang.extend(flock.favorites.bookmarksMigrator, {
    translators: {/*{{{*/
        "http://home.netscape.com/NC-rdf#Name": {
            name: "name",
            value: flock.rdf.getValue,
        },
        "http://home.netscape.com/NC-rdf#URL": {
            name: "url",
            value: flock.rdf.getValue,
        },
    },/*}}}*/
    onAssert: function (aDataSource, aSource, aProperty, aTarget) {/*{{{*/
        var id = flock.rdf.getValue(aSource);
        var pred = flock.rdf.getValue(aProperty);
        if (!this.storage[id]) this.storage[id] = {};
        
        if (pred in this.translators) {
            this.storage[id][this.translators[pred].name] = this.translators[pred].value.call(this, aTarget);
        }
    },/*}}}*/
    onAssertBatch: function (aBatch) {/*{{{*/
        var sources = aBatch.map(function (aVal) {
            return flock.rdf.getValue(aVal[1]);
        })
        var sources = flock.lang.unique(sources);
        var _this = this;
        sources.forEach(function (aSrc) {
            XXX('CHECKING', aSrc);
            if (_this.storage[aSrc].url && !_this.storage[aSrc]._committed) {
                XXX('HAS URL', aSrc, _this.storage[aSrc].url);
                if (_this.storage[aSrc].url.match(/^about:/)) return;
                
                // XXX andy: for now we are defaulting to local for imported
                _this.storage[aSrc].local = "true";
                var fav = flock.favorites.updateOrCreateFavorite(
                    _this.storage[aSrc]
                );
                if (fav && _this.importToCollection) {
                    XXX('IT IS A FAV, LET US ADD');
                    flock.favorites.addItemToCollection(
                        fav,
                        _this.importedCollection
                    );
                }
            }
        });
    },/*}}}*/
    
    get bookmarksDataSource() {/*{{{*/
        return flock.rdf.service.GetDataSource("rdf:bookmarks");
    },/*}}}*/
    importToCollection: true,
    importedCollection: "Imported Items",
    ensureImportCollection: function () {
        // ensure a collection exists to add imported items to
        flock.favorites.createCollection(this.importedCollection);
    },

    onMigrationStarted: function () {/*{{{*/
        this.ensureImportCollection();
    },/*}}}*/
    onMigrationEnded: function () {/*{{{*/

    },/*}}}*/

    // nsIObserver
    observe: function (aSubject, aTopic, aData) {/*{{{*/
        switch (aTopic) {
            case "Migration:Started":
                this.onMigrationStarted();
                break;
            case "Migration:Ended":
                this.onMigrationEnded();
                break;
        }
    },/*}}}*/

    _isAttached: false,
    attach: function () {/*{{{*/
        if (this._isAttached) return;
        this.bookmarksDataSource.AddObserver(this);
        flock.xpcom.observerService.addObserver(
            this,
            "Migration:Started",
            false
        );
        flock.xpcom.observerService.addObserver(
            this,
            "Migration:Ended",
            false
        );
        this._isAttached = true;
    },/*}}}*/
    detach: function () {/*{{{*/
        if (!this._isAttached) return;
        this.bookmarksDataSource.RemoveObserver(this);
        flock.xpcom.observerService.removeObserver(
            this,
            "Migration:Started"
        );
        flock.xpcom.observerService.removeObserver(
            this,
            "Migration:Ended"
        );
        this._isAttached = false;
    },/*}}}*/
});
