//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//


const nsIRDFRemoteDataSource        = Components.interfaces.nsIRDFRemoteDataSource;

flock.require('flock.common.dialog');

var gMapsDS = null;

// XXX because our "jax" implimentation doesn't allow adding meta data to be
// sent to the onSuccess method, are storying it in a global variable.
// this MUST be refactored as it doesn't work if adding multiple items 
// and their geocoding overlaps! 

var gAddress = null; // XXX - huge bad (not the good bad either!) hack
var gTitle = null;

var gObserver = {
    onAssert            : function(ds, source, predicate, target){
                            rebuildMap( );
                          },
    onUnassert          : function(ds, source, predicate, target){
                            rebuildMap( );
                          },
    onChange            : function(ds, source, predicate, oldTarget, newTarget){
                            rebuildMap( );
                          },
    onMove              : function(ds, oldSource, newSource, predicate, target){},
    onBeginUpdateBatch  : function(ds){},
    onEndUpdateBatch    : function(ds){}
}

function flock_mapsGetDS() {
    var RDFService = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService().QueryInterface(Components.interfaces.nsIRDFService);
	var dirService = Components.classes['@mozilla.org/file/directory_service;1'].getService(Components.interfaces.nsIProperties);
	var profileDir = dirService.get('ProfD', Components.interfaces.nsILocalFile);

    if(!gMapsDS) {
	    // Create file object in the user's profile path.
	    var file = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
	    file.initWithPath(profileDir.path);
	    file.append('flock_maps.rdf');
        if(!file.exists()) file.createUnique(0,0700);

        // Create the datasource and a sequence
        gMapsDS = new RDFDataSource('file://'+file.path);
        var root = gMapsDS.getNode("urn:flock:maps:addresses");
        root.makeSeq();
    }
    return gMapsDS;
}

function rebuild() {
	var ds = flock_mapsGetDS();
 	var ui = document.getElementById('address_list');
 	ui.database.AddDataSource(ds.getRawDataSource());
 	ui.ref = "urn:flock:maps:addresses";
 	ui.builder.rebuild();
    rebuildMap( );
}

function initMaps() {
    var ds = flock_mapsGetDS();
    var ui = document.getElementById('address_list');
    ui.database.AddDataSource(ds.getRawDataSource());
    ui.ref = "urn:flock:maps:addresses";
    ui.builder.rebuild();
    rebuildMap();
    
    var datasource = ds.getRawDataSource().QueryInterface (nsIRDFRemoteDataSource);
    datasource.AddObserver(gObserver);
}

function closeMaps() {
	var ds = flock_mapsGetDS();
    var datasource = ds.getRawDataSource().QueryInterface (nsIRDFRemoteDataSource);
    datasource.Flush();
    datasource.RemoveObserver(gObserver);
}

function _mns(str) {
    return "http://www.flock.com/rdf/maps#" + str;
}

function geocode( address, title ) {
  var req = new Ajax();
  req.onSuccess = getMapsResponse;
  gAddress = address;
  gTitle = title;
  var encodedAddress = encodeURI(address);
  XXX('address pre-encoding:',  address );
  XXX('Encoded address ', encodedAddress);
  XXX('Address ', typeof(gAddress) );
  req.request("http://api.local.yahoo.com/MapsService/V1/geocode?appid=flockmaps&location="+encodedAddress);
}

function getMapsResponse(anItem){
   XXX('address responseXML', anItem );
   var resultset = anItem.responseXML.getElementsByTagName("ResultSet")[0];
   var result = resultset.getElementsByTagName("Result")[0];

   var data = {};
   data.lat = result.getElementsByTagName("Latitude")[0].textContent;
   data.lng = result.getElementsByTagName("Longitude")[0].textContent;
   data.address = gAddress;
   data.title = gTitle;
   data.sourceurl = flock.browser.currentUrl;
   if (data.sourceurl == "about:blank")
	data.sourceurl = "";
   data.indexlabel = addressCount()+1;
   data.indexlabel = data.indexlabel.toString();
   add_object(data); 
}

function show( rdfId ) {
  var ds = flock_mapsGetDS();
  var node = ds.getNode( rdfId );
  var lat = parseFloat(node.getTarget(_mns('lat')).getValue());
  var lng = parseFloat(node.getTarget(_mns('lng')).getValue());
  $('flock-maps-minibrowser').contentWindow.gotoLatLng( lat, lng );
}

function addressCount(){
    var ds = flock_mapsGetDS();
    var root = ds.getNode("urn:flock:maps:addresses");
    // Just using getChildCount() doesn't work for some reason. Too bad for now.
    var cnt=0;
    var children = root.getChildren();
    while(children.hasMoreElements()){
      children.getNext();
      cnt++;
    }
    return cnt;
}

function rebuildMap( ) {
  var ds = flock_mapsGetDS();
  var container = ds.getNode("urn:flock:maps:addresses");
 
  var allAddresses = [];

  var nodes = container.getChildren();
  while ( nodes.hasMoreElements() ) {
    var node = nodes.getNext(); 
    var index = node.getTarget(_mns('indexlabel')).getValue();
    var address = node.getTarget(_mns('location')).getValue();
    var title = node.getTarget(_mns('title')).getValue();
    var url = node.getTarget(_mns('sourceurl')).getValue();
    var desc = node.getTarget(_mns('description')).getValue();
    var lat = parseFloat(node.getTarget(_mns('lat')).getValue());
    var lng = parseFloat(node.getTarget(_mns('lng')).getValue());
    
    if (url) {
        var body = "<a target=\"_blank\" style=\"font-weight:bold; text-decoration: underline; color: blue;\" href=\""+url+"\">"+title+"</a>";
    } else {
        var body = "<b>"+title+"</b>";
    }
    
    body += "<br/><i>"+address+"</i><br/>"+
            "<p>"+desc+"<br/>"+
            "<div style=\"text-align: right; font-size: 10px; color: blue; text-decoration: underline; padding: 0; margin: 2px 0 0 0;\"><a target='_blank' href='http://maps.yahoo.com/maps_result?addr="+encodeURI(address)+"'>view in yahoo</a></div></p>";
            
    allAddresses.push( {title: index, body: body, lat:lat, lng:lng} )
  }

  if(allAddresses.length == 0){
	    $('maps-deck').setAttribute('selectedIndex','0');
  } else {
	    $('maps-deck').setAttribute('selectedIndex','1');
  }

  $('flock-maps-minibrowser').contentWindow.updateNodes( allAddresses );
}

function maps_keypress( event ) {
  if ((event.keyCode == 8)||(event.keyCode == 46)) {
    delete_address();
  }
}

function delete_address() {
  var ds = flock_mapsGetDS();
  var container = ds.getNode("urn:flock:maps:addresses");
  var objectId = $('address_list').getItemAtIndex($('address_list').selectedIndex).id;
  var node = ds.getNode(objectId);

  ds.deleteRecursive(node);
  container.removeChild(node);
  ds.save();
}

function edit_address(){ 
    var objectId = $('address_list').getItemAtIndex($('address_list').selectedIndex).id;
    window.openDialog("chrome://browser/content/flock/maps/editAddressDlg.xul", "editAddress", "modal", objectId);
}

function add_link( node ) {
  address = node.label;
  title = flock.browser.currentTitle;
  if (address.match('maps.yahoo.com')) {
    address = address.split('?addr=')[1];
    address = address.replace( /\+at\+.*&csz=/gi, ', ');
    address = address.replace( /&csz=/gi, ', ');
    address = address.replace( /&country=/gi, ', ');
    address = address.replace( /\+/g, ' ');
    geocode( address, title );
    return;
  }

  if (address.match('maps.google.com')) {
    address = address.replace(/loc%3A/,'') // sometimes there is a 'loc:' in the query
    address = address.split('?q=')[1];
    address = address.replace( /\+/g, ' ');
    geocode( address, title );
    return;
  }
}

function add_from_input(){
        var rv = flock.dialog.prompt({
            message: "Enter Address Below",
            fields: [
                {label: "Title:", input: {
                    id: "title"
                }},
                {label: "Address:", input: {
                    id: "address"
                }}
            ],
        });
        if (!rv) return;
	geocode(rv.address, rv.title);
}

var mapsObserver={
	 onDragOver: function(event,flavor,session){
	 },

	 onDrop: function(evt,dropData,asession){
            var data=flock_getMarkedUpSelection(evt, dropData, asession);
            var title = flock.browser.currentTitle;
	    XXX('dropdata', data.type);
	    switch(data.type){
             case "link": add_link(data);break;
             case "document":
                XXX('address pre-geocoding:', data.label );
		geocode(data.label.toString(), title);
		break;
             default: XXX('datatype', data.type);
            }
	 },

	 onDragStart: function(event, transferData, action){
			dump('Drag started!');

			transferData.data=new TransferData();
			transferData.data.addDataForFlavour("text/html",'html');
			transferData.data.addDataForFlavour("text/x-moz-url",'url');
			transferData.data.addDataForFlavour("text/unicode",'unicode');
			transferData.data.addDataForFlavour("application/x-moz-file",'file');
	 },

	 getSupportedFlavours: function(){
			flavors=new FlavourSet();
			//flavors.appendFlavour('flock/shelfobject');
			flavors.appendFlavour('text/unicode');
			flavors.appendFlavour('text/html');
			flavors.appendFlavour('text/x-moz-url');
			flavors.appendFlavour('application/x-moz-file','nsIFile');
			return flavors;
	 }
}

function createLink(rdfId){
  var ds = flock_mapsGetDS();
  var node = ds.getNode( rdfId );
  var address = node.getTarget(_mns('location')).getValue();
  var encodedAddress = encodeURI(address); 
  var theUrl = "http://maps.yahoo.com/maps_result?addr="+encodedAddress;
  var result = "<a href='"+theUrl+"'>"+address+"</a>";
  return result;
}

var addressObserver={
	 onDragStart: function(event, transferData, action){
		try{
			/*
			for(i in event.target.session){
				dump('==> ET: ' + i + ': ' + event.target.session[i] + '\n');
			}
			dump('SOURCE NODE: ' + event.target.sourceNode + '\n');
			*/
                        src=createLink(event.target.id);
			dump('DRAGGING SOURCE: ' + src + '\n');
			transferData.data=new TransferData();
			transferData.data.addDataForFlavour("text/html",src);
			transferData.data.addDataForFlavour("text/unicode",src);
		}
		catch(e){
			XXX('addressobserver error', e);
		}
	},
	onClick: function( event ) {
		alert(event);
	}
}

function add_object(data){
	try{
	    var ds = flock_mapsGetDS();
	    var container = ds.getNode("urn:flock:maps:addresses");
	    var date=new Date();

	    var id='obj_' + date.getTime();

	    var root = ds.getNode("urn:flock:maps:addresses");
	    var node = ds.getNode("urn:flock:maps:addresses:" + id);

  	    XXX('Node: ', node);
	    XXX('Root: ', root);

	    node.addTargetOnce(_mns('sourceurl'), data.sourceurl + "" );
	    node.addTargetOnce(_mns('location'), data.address + "");
	    node.addTargetOnce(_mns('indexlabel'), data.indexlabel +"");
	    node.addTargetOnce(_mns('lat'), data.lat + "" );
	    node.addTargetOnce(_mns('lng'), data.lng + "" );
	    node.addTargetOnce(_mns('title'), data.title + "" );
	    node.addTargetOnce(_mns('description'), (data.description || "") );

	    root.addChild(node);
	    ds.save();  
	    show('urn:flock:maps:addresses:'+id);
	}
	catch(e){
		XXX('add_object error', e);
	}
}

// The following functions are called in the scope of editAddressDlg.xul

function fillFields() {
   if (window.arguments) {
      addressId = window.arguments;
      var ds = flock_mapsGetDS();
      var node = ds.getNode( addressId );

      $('address-address').value = node.getTarget(_mns('location')).getValue();
      $('address-title').value = node.getTarget(_mns('title')).getValue();
      $('address-url').value = node.getTarget(_mns('sourceurl')).getValue();
      $('address-description').value = node.getTarget(_mns('description')).getValue();
   }
}

function saveAddress() {
   addressId = window.arguments;
   var ds = flock_mapsGetDS();
   var root = ds.getNode("urn:flock:maps:addresses");
   var node = ds.getNode( addressId );

   node.addTargetOnce(_mns('location'), $('address-address').value);
   node.addTargetOnce(_mns('title'), $('address-title').value);
   node.addTargetOnce(_mns('sourceurl'), $('address-url').value);
   node.addTargetOnce(_mns('description'), $('address-description').value);

   if (root.getChildIndex(node) == -1){
      root.addChild(node);
   }

   $('address-title').value = node.getTarget(_mns('title')).getValue();
   $('address-url').value = node.getTarget(_mns('sourceurl')).getValue();
   $('address-description').value = node.getTarget(_mns('description')).getValue();

   ds.save();  
}

