/*
     BEGIN FLOCK GPL
     
     Copyright Flock Inc. 2005
     http://flock.com
     
     This file may be used under the terms of of the
     GNU General Public License Version 2 or later (the "GPL"),
     http://www.gnu.org/licenses/gpl.html
     
     Software distributed under the License is distributed on an "AS IS" basis,
     WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
     for the specific language governing rights and limitations under the
     License.
     
     END FLOCK GPL
*/

flock.require('flock.photo.photoCommon');

var gPageSize = 50;
var gFinishedQuery = false;
var gBrowseResult = null;

function flock_photoOpenProperties() {
    var selection = $("flock-photo-upload-slideybox").getSelection();
    if(selection && selection.length) {
        window.openDialog("photoProperties.xul", "_blank", "modal,close,centerscreen", selection);
    }
}

function flock_photoRemove() {
    var selection = $("flock-photo-upload-slideybox").getSelection();
    if(selection && selection.length) {
        for (var i = 0; i < selection.length; i++) {
	        selection[i].firstChild.removeMe();
        }
    }
}

////////////////////////////////////////////////////////////////////////////////////////
// Upload stuff
////////////////////////////////////////////////////////////////////////////////////////

const FLOCK_IO_CHUNKSIZE = 65536 * 4;
const FLOCK_THUMB_SIZE = 75;
const FLOCK_PREVIEW_SIZE = 300;
const FLOCK_THUMB_EXT = ".thumb.jpg";
const FLOCK_PREVIEW_EXT = ".preview.jpg";
var gPhotoImportQ = new Array();
var gAPI = null;
var gAuthenticatedUser = null;

function flock_photoUploadTopbarLoad() {
	window.flock_photoUpload = flock_photoUpload;
    flock.photo.service.addListener(UploadListener);
    flock_photoUpdateEnabledness();
    $("flock-photo-upload-slideybox").setListener(PhotoUploadScrolleyboxListener);
    //XXX("---------------------------------------------- photo upload topbar loaded", svc )
}

function flock_photoUploadTopbarUnload() {
	flock_photoCommonUnload();
    flock.photo.service.removeListener(UploadListener);
}

function flock_photoLoadTimer() {
    if(gPhotoImportQ.length==0) return;
    window.setTimeout("flock_photoLoadTimer()", 100);
    var requestObserver = gPhotoImportQ.pop();
    requestObserver.asyncCopier.asyncCopy(requestObserver, requestObserver.sinkFile);
}

////////////////////////////////////////////////////////////////////////////////////////
// UI Listeners
////////////////////////////////////////////////////////////////////////////////////////

var PhotoUploadScrolleyboxListener = {
    onDrop: function(aHoverChild, aSelection) {
        dump("onDrop");
        try {
        var svc = flock.photo.service;
        var hoverChild = aHoverChild;
        if(hoverChild) hoverChild = svc.getPhotoUpload(hoverChild.firstChild.id);
        for(var i=0;i<aSelection.length;++i) {
            var photo = aSelection[i].firstChild;
            dump("Adding photo upload " + photo.id + "\n");
            if(aHoverChild && aHoverChild.firstChild==photo) continue;
            svc.addPhotoUpload(hoverChild, svc.getPhotoUpload(photo.id));
        }
        }
        catch(e) {
            alert(e);
        }
    },
    onScrollRight: function() {
    },
    onScrollLeft: function() {
    },
    onRemoveSelection: function(aSelection) {
        var svc = flock.photo.service;
        for(var i=0;i<aSelection.length;++i) {
            var child = aSelection[i].firstChild;
            dump(aSelection.length + " " + aSelection[i].nodeName + " " + child.nodeName + " onRemove Selection\n");
            svc.removePhotoUpload(child);
        }
    }
}

var UploadListener = {
    onUploadStart: function (aCurrentUpload, aCount, aMax) {
        flock_photoUpdateEnabledness();
    },
    onUpload: function (aCurrentUpload, aCount, aMax) {
        flock_photoUpdateEnabledness();
    },
    onUploadAdd: function (aCurrentUpload) {
        flock_photoUpdateEnabledness();
    },
    onUploadRemove: function () {
        flock_photoUpdateEnabledness();
    },
    onUploadComplete: function () {
        flock_photoUpdateEnabledness();
    },
    onUploadError: function (aUpload, aCount, aMax, aError) {
        // LOCALIZATION NOTE: localize me
        alert("There was an error uploading your photos.  Please try again later.");
        flock_photoUpdateEnabledness();
    },
    onAuthStateChange: function (aState) {
        flock_photoUpdateEnabledness();
    }
}


////////////////////////////////////////////////////////////////////////////////////////
// Drag/Drop stuff
////////////////////////////////////////////////////////////////////////////////////////


var photoTopbarDNDObserver = new DNDObserver();

photoTopbarDNDObserver.canHandleMultipleItems = true;

photoTopbarDNDObserver.onDragEnter = function (aEvent, aXferData, aDragSession) { }

photoTopbarDNDObserver.onDragOver = function (aEvent, aXferData, aDragSession) { }

photoTopbarDNDObserver.isAPhoto = function (aURL) {
    if(aURL.indexOf("file")!=0) return false;
    var lc = aURL.toLowerCase();
    if(lc.match(/.+jpg$/)) return true;
    if(lc.match(/.+jpeg$/)) return true;
    if(lc.match(/.+png$/)) return true;
    if(lc.match(/.+gif$/)) return true;
    return false;
}

photoTopbarDNDObserver.onDrop = function (aEvent, aXferData, aDragSession) {
    var svc = flock.photo.service;
    var hoverChild = $("flock-photo-upload-slideybox").hoverChild;

    //hack to handle weird template issues
    if(hoverChild) hoverChild = svc.getPhotoUpload(hoverChild.firstChild.id);

    for(var i=0;i<aXferData.dataList.length;++i) {
        var supports = aXferData.dataList[i].dataList[0].supports;
        var url;
        try {
            var file = supports.QueryInterface(Components.interfaces.nsIFile);
            url = flock_getURLFromFile(file);
        }
        catch(e) {
            url = supports.QueryInterface(Components.interfaces.nsISupportsString);
        }
        url = url + "";
        if(!this.isAPhoto(url)) continue;

        var sourceFile = flock_getFileFromURL(url);
        var sourceBufferStream =  flock_getBufferredInputStream(sourceFile);

        var sinkFileName = flock.photo.util.getUniqueFname(url);

        var sinkFile = flock_getTmpFile(sinkFileName);
        var sinkBufferStream = flock_getBufferredOutputStream(sinkFile);

        var asyncCopier = Components.classes["@mozilla.org/network/async-stream-copier;1"]
                .createInstance(Components.interfaces.nsIAsyncStreamCopier);

        asyncCopier.init(sourceBufferStream, sinkBufferStream, null, true, true, FLOCK_IO_CHUNKSIZE);

        var requestObserver = {
            fname: sourceFile.leafName,
            asyncCopier: asyncCopier,
            url: url,
            hoverChild: hoverChild,
            sinkFile: sinkFile,
            sinkFileName: sinkFileName,
            onStartRequest: function(aRequest, aContext) {
            },
            onStopRequest: function(aRequest, aContext) {
                var sourcePath = this.sinkFile.path;
                var thumbPath = sourcePath + FLOCK_THUMB_EXT;
                var previewPath = sourcePath + FLOCK_PREVIEW_EXT;

//                var imageUtils = flock_getImageUtils();

                var dims = FLOCK_THUMB_SIZE + "x" + FLOCK_THUMB_SIZE;
                var dimsX2 = 4*FLOCK_THUMB_SIZE + "x" + 4*FLOCK_THUMB_SIZE;
                //var args= flock.photo.util.createArgs("convert", "-size", dims, sourcePath, "-resize", dims, thumbPath);
                //-size 300x300 hatching.jpg  \
                //           -thumbnail x200   -resize '200x<'   -resize 50% \
                //                      -gravity center -crop 100x100+0+0  +repage   cut_to_fit.gif
//                -resize x160 -resize '160x<'   -resize 50% \
//                           -gravity center  -crop 80x80+0+0 +repage  space_fill.jpg
//                var args= flock.photo.util.createArgs("convert", "-size", dimsX2, sourcePath, "-resize", dimsX2, 
//                                    "-resize", "50%", "-gravity", "center", "-crop", dims+"+0+0", thumbPath);
                //var args= flock.photo.util.createArgs("convert", sourcePath, "-resize", "x" + FLOCK_THUMB_SIZE*2, "-resize",  FLOCK_THUMB_SIZE*2 + "x<", 
                 //                   "-resize", "50%", "-gravity", "center", "-crop", dims + "+0+0",  thumbPath);
                //imageUtils.resize(sourcePath, thumbPath, FLOCK_THUMB_SIZE, FLOCK_THUMB_SIZE);
                //imageUtils.doMagick(args, args.length);
                flock.photo.util.resize(FLOCK_PREVIEW_SIZE, sourcePath, previewPath);
                flock.photo.util.thumb(FLOCK_THUMB_SIZE, previewPath, thumbPath);

                var thumbFile = flock_getTmpFile(this.sinkFileName + FLOCK_THUMB_EXT);
                var thumbSpec = flock_getURLFromFile(thumbFile);

                var previewFile = flock_getTmpFile(this.sinkFileName + FLOCK_PREVIEW_EXT);
                var previewSpec = flock_getURLFromFile(previewFile);
                //alert(previewSpec + " " + previewPath);

                var date = new Date();
                var photo = {};
                photo.originalFileSpec = flock_getURLFromFile(this.sinkFile);
                photo.originalFilePath = sourcePath;
                photo.thumbFileSpec = thumbSpec;
                photo.thumbFilePath = thumbPath;
                photo.previewFileSpec = previewSpec;
                photo.previewFilePath = previewPath;
                photo.is_public = "true";
                photo.tags = "";
                photo.description = "";
                photo.is_friend = "false";
                photo.is_family = "false";
                photo.title = this.fname;
                photo.state = "";
                photo.rotation = "0";
                photo.id = "uri:" + hex_md5(sourcePath);
                //if(this.hoverChild) dump(this.hoverChild.nodeName + " ADDING\n");
                svc.addPhotoUpload((this.hoverChild?this.hoverChild:null),photo);
            }
        };
        gPhotoImportQ.unshift(requestObserver);
    }
    flock_photoLoadTimer();
}

photoTopbarDNDObserver.getSupportedFlavours = function () {
    var flavourSet = new FlavourSet();
    flavourSet.appendFlavour("text/x-moz-url");
    flavourSet.appendFlavour("application/x-moz-url");
    flavourSet.appendFlavour("application/x-moz-file");
    return flavourSet;                           
}

////////////////////////////////////////////////////////////////////////////////////////
// Assorted UI
////////////////////////////////////////////////////////////////////////////////////////

function flock_photoWeHavePhotos() {
    if($("flock-photo-upload-slideybox").childNodes.length<2) return true;
    return false;
}

function flock_photoUpdateEnabledness() {
    var svc = flock.photo.service;
    if(svc.isUploading()) {
        $$("flock-photo-upload-button").setAttribute("label", "Cancel Upload");
        $$("flock-photo-upload-button").setAttribute("oncommand", "TopbarJSCtx().flock_photoUploadCancel()");
        flock_photoUploadDisable();
    }
    else {
        $$("flock-photo-upload-button").setAttribute("label", "Upload");
        $$("flock-photo-upload-button").setAttribute("oncommand", "TopbarJSCtx().flock_photoUpload()");
        flock_photoUploadEnable();
    }

    if(flock_photoWeHavePhotos()) {
        $("flock-photo-instructions").removeAttribute("hidden");
        $$("flock-photo-upload-message").setAttribute("hidden", true);
    }
    else {
        $("flock-photo-instructions").setAttribute("hidden", true);
        $$("flock-photo-upload-message").removeAttribute("hidden");
        var photo_count = $("flock-photo-upload-slideybox").childNodes.length - 1;
        if (photo_count > 1) {
            $$("flock-photo-upload-message").value = "You have " + photo_count + " photos to upload.";
        } else {
            $$("flock-photo-upload-message").value = "You have 1 photo to upload.";
        }
    }

    if(!svc.api.isLoggedIn()) {
        flock_photoUploadDisableButton();
    }
    else if(flock_photoWeHavePhotos()) {
        flock_photoUploadDisableButton();
    }
    else {
        flock_photoUploadEnableButton();
    }
}

function flock_photoUploadEnable() {
    $("flock-photo-upload-slideybox").enabled = true;
    $("flock-photo-disable").setAttribute("hidden", true);
}

function flock_photoUploadEnableButton() {
    $$("flock-photo-upload-button").setAttribute("uploading", false);
    $$("flock-photo-upload-button").removeAttribute("disabled");
}

function flock_photoUploadDisableButton() {
    $$("flock-photo-upload-button").setAttribute("uploading", true);
    $$("flock-photo-upload-button").setAttribute("disabled", true);
}

function flock_photoUploadDisable() {
    $("flock-photo-upload-slideybox").enabled = false;
    $("flock-photo-disable").removeAttribute("hidden");
    flock_photoUploadDisableButton();
}

function flock_photoUpload() {
    if($("flock-photo-upload-slideybox").enabled == true) { //that check isn't really valid anymore
        $("flock-photo-upload-slideybox").scrollHome();
        flock_photoUploadDisable();
        flock_photoDoUpload();
    } else {
        //gUploadListener.running = false;
        flock_photoUploadEnable();
    }
}

function flock_photoUploadCancel() {
    flock.photo.service.cancelUpload()
}

function flock_photoDoUpload() {
    var svc = flock.photo.service;
/*
    var loginListener = new PhotoLoginListener();
    loginListener.postAuthFunc = function () {
        var photo = $("flock-photo-upload-slideybox").firstChild.nextSibling;//the first is a template
        if(photo) {
            svc.upload();
        }
        else {
            flock_photoUploadEnable();
            XXX("All photos have been uploaded");
        }
    }
*/
    if(svc.isLoggedIn()) {
        svc.upload();
    }
/*
    else {
        svc.login(loginListener);
    }
*/
}
