// vim: ts=4 sw=4 cindent expandtab
//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//


function flock_services_prefs_init () {
    flock_services_prefs_favorites_init ();
    flock_photoPrefLoad ();
}

const PREFS = Components.classes["@mozilla.org/preferences-service;1"]
        .getService(Components.interfaces.nsIPrefBranch);
const FAVES = Components.classes["@mozilla.org/rdf/datasource;1?name=flock-favorites"].getService (Components.interfaces.flockIFavoritesService);

function flock_services_prefs_favorites_init () {
    // update checkboxes
    $("webServiceEnabled").checked = FAVES.getWebServiceEnabled ();
    try {
        $("addAndTagByDefault").checked = 
            PREFS.getBoolPref ('flock.favorites.addAndTagByDefault');
    } catch (e) { }
    try {
        $("shareByDefault").checked = 
            PREFS.getBoolPref ('flock.favorites.shareByDefault');
    } catch (e) { }  
    try {
        var isPrivate = (FAVES.webService.supportedFields & Components.interfaces.flockIFavoriteProperties.FIELD_PRIVATE);
        $("shareByDefault").hidden = (isPrivate ? false : true);
    } catch (e) { }
    // update the menulist
    var menulist = $('webServiceList');
    var selected = FAVES.webService;

    // empty the menulist
    menulist.removeAllItems ();

    // add things to the menulist
    var svcs = FAVES.getWebServices ();
    while (svcs.hasMoreElements ()) {
        var svc = svcs.getNext ();
        svc.QueryInterface (Components.interfaces.flockIFavoritesWebService);
        var id = svc.id;
        var item = menulist.appendItem (svc.name, id);
        if (id == selected.id) {
            menulist.selectedItem = item;
        }
    }
}

function flock_webServiceChange () {
    var menulist = $('webServiceList');
    if (menulist.selectedItem.value == FAVES.webService.id) {
        // no change
        return;
    }

    if (confirm ('You are switching to a different online favorites service. Flock will move all ' +
                'your current online favorites to your new service. If your new service does not ' +
                'support private online favorites, any private online favorites you have will ' +
                'become public.\n\n' + 
                'Flock will restart automatically so the change can take effect. Open windows ' +
                'will be closed.\n\n' +
                'If you want to move your online favorites to your new service, and restart ' +
                'Flock, click OK.')) {
        // make the prefs changes
        PREFS.setCharPref ('flock.favorites.webservice.id', 
                menulist.selectedItem.value);
        PREFS.setBoolPref ('flock.favorites.uploadAll', true);

        // restart the browser
        Components.classes['@mozilla.org/toolkit/app-startup;1']
            .getService(Components.interfaces.nsIAppStartup)
            .quit (Components.interfaces.nsIAppStartup.eRestart |
                    Components.interfaces.nsIAppStartup.eAttemptQuit);
    } else {
        flock_services_prefs_favorites_init ();
    }
}

function flock_setWebServiceEnabled () {
    FAVES.setWebServiceEnabled ($('webServiceEnabled').checked);
    PREFS.setBoolPref ('flock.favorites.askedAboutSharing', true);
    if ($('webServiceEnabled').checked) {
        FAVES.synchronize ();
    }
}

function flock_setAddAndTagByDefault() {
    PREFS.setBoolPref ('flock.favorites.addAndTagByDefault', 
            $("addAndTagByDefault").checked);
}

function flock_setShareByDefault() {
    PREFS.setBoolPref ('flock.favorites.shareByDefault', 
            $("shareByDefault").checked);
}

function flock_photoPrefLoad() {
    var pref = flock_getCharPref(FLOCK_PREF_PHOTO_API_UINAME);
    $("flockPhotoServicesMenuList").value = pref;
    var resize = flock_getBoolPref(FLOCK_PREF_PHOTO_RESIZE);
    if(resize=="" || resize==null) {
        flock_setBoolPref(FLOCK_PREF_PHOTO_RESIZE, false);
        resize = flock_getBoolPref(FLOCK_PREF_PHOTO_RESIZE);
    }
    $("flockPhotoResizeRadioGroup").value = resize;

    var maxDim = flock_getIntPref(FLOCK_PREF_PHOTO_RESIZE_DIMENSIONS);
    $("flockPhotoQualityList").value = maxDim;
    flock_photoPrefSetConfigEnabled();
}

function flock_photoPrefSetConfigEnabled() {
    var item = $("flockPhotoServicesMenuList").selectedItem;
    var configUrl =  item.getAttribute("apiconfigurl");
    if(configUrl && configUrl.length) {
        $("flockPhotoConfigButton").removeAttribute("disabled");
    }
    else {
        $("flockPhotoConfigButton").setAttribute("disabled", true);
    }
}

function flock_photoPrefSetService(aObj) {
    var item =  aObj.selectedItem;
    var uiname =  item.getAttribute("value");
    var classname =  item.getAttribute("apiclassname");
    var jsfile =  item.getAttribute("apijsfile");
    var cid =  item.getAttribute("apicid");
    flock_photoPrefSetConfigEnabled();

    flock_photoSetAPI(uiname, classname, jsfile, cid)
}

function flock_photoPrefSetResize(aObj) {
    flock_setBoolPref(FLOCK_PREF_PHOTO_RESIZE, aObj.value=="true");
    flock_setIntPref(FLOCK_PREF_PHOTO_RESIZE_DIMENSIONS, $("flockPhotoQualityList").value);
    if(aObj.value=="true") {
        $("flockPhotoQualityList").removeAttribute("disabled");
    }
    else {
        $("flockPhotoQualityList").setAttribute("disabled",  true);
    }
}

function flock_photoPrefSetQuality(aValue) {
    flock_setIntPref(FLOCK_PREF_PHOTO_RESIZE_DIMENSIONS, aValue);
}

function flock_photoPrefConfigure() {
    var item = $("flockPhotoServicesMenuList").selectedItem;
    var url = item.getAttribute("apiconfigurl");
    window.openDialog(url,"photoComfig", "modal");
}
