// vim: ts=4 sw=4 expandtab cindent foldmethod=marker
//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

// CONSTANTS {{{
const flockIFavoritesWebService = Components.interfaces.flockIFavoritesWebService;
const flockIFavoritesChange = Components.interfaces.flockIFavoritesChange;
const flockIFavoritesChangeFavorite = Components.interfaces.flockIFavoritesChangeFavorite;
const flockIFavoritesQueue = Components.interfaces.flockIFavoritesQueue;
const flockIFavoritesUtils = Components.interfaces.flockIFavoritesUtils;
const flockIFavoritesService = Components.interfaces.flockIFavoritesService;
const flockIFavoriteProperties = Components.interfaces.flockIFavoriteProperties;
const nsISupports = Components.interfaces.nsISupports;
const nsITimer = Components.interfaces.nsITimer;
const nsITimerCallback = Components.interfaces.nsITimerCallback;
const nsIXMLHttpRequest = Components.interfaces.nsIXMLHttpRequest;
const nsIPasswordManager = Components.interfaces.nsIPasswordManager;
const nsIPassword = Components.interfaces.nsIPassword;
const nsIDOMSerializer = Components.interfaces.nsIDOMSerializer;
const nsIPrefBranch              = Components.interfaces.nsIPrefBranch;
const nsIPromptService = Components.interfaces.nsIPromptService;

const TIMER_CONTRACTID = '@mozilla.org/timer;1';
const XMLHTTPREQUEST_CONTRACTID = '@mozilla.org/xmlextras/xmlhttprequest;1';
const PASSWORDMANAGER_CONTRACTID = '@mozilla.org/passwordmanager;1';
const XMLSERIALIZER_CONTRACTID = '@mozilla.org/xmlextras/xmlserializer;1';
const FFS_CONTRACTID = '@mozilla.org/rdf/datasource;1?name=flock-favorites';
const PREFERENCES_CONTRACTID = '@mozilla.org/preferences-service;1';
const PROMPT_CONTRACTID = '@mozilla.org/embedcomp/prompt-service;1';

const PREFS = Components.classes[PREFERENCES_CONTRACTID].getService (nsIPrefBranch);

const DIRECTION_TO_WEB_SERVICE = flockIFavoritesQueue.DIRECTION_TO_WEB_SERVICE;
const DIRECTION_FROM_WEB_SERVICE = flockIFavoritesQueue.DIRECTION_FROM_WEB_SERVICE;

const FIELD_WSID                 = flockIFavoriteProperties.FIELD_WSID;
const FIELD_TAGS                 = flockIFavoriteProperties.FIELD_TAGS;
const FIELD_PERMISSIONS          = flockIFavoriteProperties.FIELD_PERMISSIONS;
const FIELD_NAME                 = flockIFavoriteProperties.FIELD_NAME;
const FIELD_SHORTCUTURL          = flockIFavoriteProperties.FIELD_SHORTCUTURL;
const FIELD_DESCRIPTION          = flockIFavoriteProperties.FIELD_DESCRIPTION;
const FIELD_NSYNC                = flockIFavoriteProperties.FIELD_NSYNC;
const FIELD_LASTVISITDATE        = flockIFavoriteProperties.FIELD_LASTVISITDATE;
const FIELD_LASTMODIFIEDDATE     = flockIFavoriteProperties.FIELD_LASTMODIFIEDDATE;
const FIELD_ADDEDDATE            = flockIFavoriteProperties.FIELD_ADDEDDATE;

const TIMEOUT = 10; // cancel requests that take more than 10 seconds 

const SEND_WAIT = 1; // wait one second between sending updates
const RECV_WAIT = 60*5; // wait 

const PREF_USERNAME = 'bokee.username';
const PREF_PASSWORD = 'bokee.password';
/// }}}

// The bokeeWebService flockIFavoritesWebService implementation {{{
function bokeeWebService () {/*{{{*/
    dump ('constructed bokeeWebService\n');
    this.mQueue = null;

    this.mBokee = new flockXmlRpcServer ('http://blogmark.bokee.com/xml-rpc');

    this.mLastUpdate = 0;

    this.mInitialized = false;

    this.mFlockUserType = null;

    this.mSendTimer = Components.classes[TIMER_CONTRACTID]
        .createInstance(nsITimer);
    this.mSendTimer.initWithCallback (this, SEND_WAIT*1000, 
            nsITimer.TYPE_REPEATING_SLACK);

    // set attributes
    // FIXME: need to know how to find users' own page
    this.webView = 'http://blogmarks.bokee.com/';
    this.configurationDialog = null;
    this.name = 'Bokee';
    this.description = 'Interacts with the Bokee.com Blogmarks service';
    this.id = 'bokee';
    this.topbarPage = null;
    this.supportedFields = FIELD_WSID|FIELD_NAME;
    this.supportsCollections = false;
    this.supportsExplicitDeletes = false;
}/*}}}*/
// private methods
bokeeWebService.prototype.call = function (aMethod, aArgs, aListener) { // {{{
    var un = this.username ();
    var pw = this.password ();

    var real_args = [un, pw];
    for (var i in aArgs) {
        real_args.push (aArgs[i]);
    }

    dump ('bokeeWebService.call: with args: '+real_args+'\n');
    this.mBokee.call (aMethod, real_args, aListener);
} // }}}
bokeeWebService.prototype.promptForLogin = function () { // {{{
    var svc = Components.classes[PROMPT_CONTRACTID]
        .getService (nsIPromptService);
    var un = '';
    var pw = '';
    try {
        un = PREFS.getCharPref (PREF_USERNAME);
    } catch (e) { }
    try {
        pw = PREFS.getCharPref (PREF_PASSWORD);
    } catch (e) { }
    var u = { value: un }
    var p = { value: pw }
    var ok = svc.promptUsernameAndPassword (null, 'Bokee Login Details', 
            'Enter your Bokee user name and password', u, p, null, {value:0});
    if (!ok) {
        u.value = '';
        p.value = '';
    }
    PREFS.setCharPref (PREF_USERNAME, u.value);
    PREFS.setCharPref (PREF_PASSWORD, p.value);
} // }}}    
bokeeWebService.prototype.username = function () { // {{{
    var un = null;
    while (un == null) {
        // try to get the preference
        try {
            un = PREFS.getCharPref (PREF_USERNAME);
        } catch (e) { }

        if (un == null) {
            this.promptForLogin ();
        }
    }
    return un;
} // }}}
bokeeWebService.prototype.password = function () { // {{{
    var pw = null;
    while (pw == null) {
        // try to get the preference
        try {
            pw = PREFS.getCharPref (PREF_PASSWORD);
        } catch (e) { }

        if (pw == null) {
            this.promptForLogin ();
        }
    }
    return pw;
} // }}}
bokeeWebService.prototype.initialize = function () { // {{{
    var svc = this;

    // get all categories
    var listener = {}
    listener.onResult = function (aResult) {
        dump ('onResult: userType.getUserTypes ('+aResult+')\n');
        svc.mFlockUserType = null;
        for (var i in aResult) {
            dump ('userType: '+aResult[i].typeName+' : '+aResult[i].typeID+'\n');

            if (aResult[i].typeName == 'Flock') {
                // if we find a "Flock" userType remember it
                svc.mFlockUserType = aResult[i].typeID;
                break;
            }
        }

        // if we've found no flock userType lets create one
        if (svc.mFlockUserType == null) {
            var listener = {}
            listener.onResult = function (aResult) {
                dump ('onResult: userType.addUserType ('+aResult+')\n');
                svc.mFlockUserType = aResult.mTypeID;
                svc.mInitialized = true;
            }
            listener.onFault = function (aFaultCode, aFaultString) {
                dump ('onFault: userType.addUserType ('+aFaultCode+', '+aFaultString+')\n');
            }
            listener.onError = function (aMessage) {
                dump ('onError: userType.addUserType ('+aMessage+'\n');
            }
            svc.call ('userType.addUserType', ["Flock"], listener);
            return;
        }

        // else
        svc.mInitialized = true;
        // force a sync
        var fsvc = Components.classes[FFS_CONTRACTID]
            .getService (flockIFavoritesService);
        fsvc.synchronize ();
    }
    listener.onFault = function (aFaultCode, aFaultString) {
        dump ('onFault: userType.getUserTypes ('+aFaultCode+', '+aFaultString+')\n');
        // BOKEE WORKAROUND
        if (aFaultCode == 0 && aFaultString == 'java.lang.IllegalArgumentException: null values not supported by XML-RPC') {
            // bokee sends a fault when there are no userTypes
            this.onResult ([]);
        }
    }
    listener.onError = function (aMessage) {
        dump ('onError: userType.getUserTypes ('+aMessage+'\n');
    }
    this.call ('userType.getUserTypes', [], listener);
    // if there's no Flock category:
    //     create one
} // }}}
// nsISupports implementation
bokeeWebService.prototype.QueryInterface = function (iid) {/*{{{*/
    if (iid.equals(flockIFavoritesWebService) || 
            iid.equals(nsITimerCallback) ||
            iid.equals(nsISupports)) {
        return this;
    }

    throw Components.results.NS_ERROR_NO_INTERFACE;
}/*}}}*/
// nsITimerCallback implementation
bokeeWebService.prototype.notify = function (aTimer) {//{{{
    if (aTimer == this.mSendTimer) {
        if (!this.mInitialized) {
            return;
        }
        if (!this.mQueue) {
            return;
        }

        var queue = this.mQueue;

        var change = this.mQueue.nextChange (DIRECTION_TO_WEB_SERVICE);
        if (change == null) {
            // no change pending
            return;
        }

        try {
            change = change.QueryInterface (flockIFavoritesChangeFavorite);
            if (change == null) {
                // not the right kind of change
                return;
            }
        } catch (e) {
            // not the right kind of change
            return;
        }

        var listener = new Object ();
        listener.onResult = function (aResult) {
            // ok
            // FIXME: handle {'checkUser': 1, 'keyLinkID': -1}
            //        this means that there was an conflict, I think
            if (!(change.properties.fields & FIELD_WSID) &&
                    aResult.keyLinkID > 0) {
                // we didn't have the ID but now we do.
                queue.changeFavorite (change.url,
                        { wsid: aResult.keyLinkID, fields: FIELD_WSID },
                        DIRECTION_FROM_WEB_SERVICE);
            }
        }
        listener.onFault = function (aFaultCode, aFaultString) {
            // re-queue
            if (change.isDelete) {
                queue.deleteFavorite (change.url, change.properties,
                        DIRECTION_TO_WEB_SERVICE);
            } else {
                queue.changeFavorite (change.url, change.properties,
                        DIRECTION_TO_WEB_SERVICE);
            }
        }
        listener.onError = function (aMessage) {
            // re-queue
            if (change.isDelete) {
                queue.deleteFavorite (change.url, change.properties,
                        DIRECTION_TO_WEB_SERVICE);
            } else {
                queue.changeFavorite (change.url, change.properties,
                        DIRECTION_TO_WEB_SERVICE);
            }
        }

        if (change.isDelete) {
            dump ('bokee delete: '+change.url+'\n');
            dump ('fields&FIELD_WSID: '+change.properties.fields&FIELD_WSID+'\n');
            dump ('wsid: '+change.properties.wsid+'\n');
            if (!change.properties.wsid) {
                // can't do much without one of these...
                return;
            }
            this.call ('keyLink.deleteKeyLink', 
                    [parseInt (change.properties.wsid)], 
                    listener);
        } else {
            if (change.properties.fields & FIELD_WSID) {
                dump ('bokee change: '+change.url+'\n');
                dump ('wsid: '+change.properties.wsid+'\n');
                this.call ('keyLink.updateKeyLink', 
                        [parseInt (change.properties.wsid),
                        change.properties.name, 'NORMAL', this.mFlockUserType,
                        '', '', '', 0], listener);
            } else {
                dump ('bokee add: '+change.url+'\n');
                dump (' name='+change.properties.name+'\n');
                this.call ('keyLink.addKeyLink', [change.properties.name, 
                        change.url, 'NORMAL', this.mFlockUserType, '', '', '', 
                        0], listener);
            }
        }

    } else {
        dump ('callback for unknown timer: '+aTimer+'\n');
    }
}//}}}


// flockIFavoritesWebService implementation
bokeeWebService.prototype.setFavoritesQueue = function (aQueue) {/*{{{*/
    dump ('bokeeWebService.setFavoritesQueue('+aQueue+')\n');
    if (!this.mInitialized && aQueue) {
        this.initialize ();
    }
    this.mQueue = aQueue;
}/*}}}*/
bokeeWebService.prototype.getAllFavorites = function (aListener) {/*{{{*/
    dump ('bokeeWebService.getAllFavorites('+aListener+')\n');
    if (!this.mInitialized) {
        aListener.onFailed ('not initialized');
        return;
    }

    var changeqi = function (aIID) {
        if (aIID.equals(flockIFavoritesChange) || 
                aIID.equals(flockIFavoritesChangeFavorite) || 
                aIID.equals(nsISupports)) {
            return this;
        }
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }

    var listener = {};
    listener.onResult = function (aResult) {
        dump ('bokeeWebService.getAllFavorites.listener.onResult: '+aResult+'\n');
        var changes = new Array ();
        for (var i=0; i<aResult.length; i++) {
            var blogmark = aResult[i];
            dump ('BlogMark: '+blogmark.linkURL+'\n');
            dump ('  title: '+blogmark.linkTitle+'\n');
            dump ('   wsid: '+blogmark.linkID+'\n');
            var change = new Object ();
            change.url = blogmark.linkURL;
            change.isDelete = false;
            change.properties = new Object ();
            change.properties.wsid = blogmark.linkID;
            change.properties.name = blogmark.linkTitle;
            change.properties.fields = FIELD_WSID|FIELD_NAME;
            change.QueryInterface = changeqi;
            changes.push (change);
        }
        changes.hasMoreElements = function () { return this.length != 0; }
        changes.getNext = function () { return this.shift (); }
        aListener.onSucceeded (changes);
    }
    listener.onFault = function (aFaultCode, aFaultString) {
        aListener.onFailed ('fault: '+aFaultString);
    }
    listener.onError = function (aMessage) {
        aListener.onFailed ('error: '+aMessage);
    }

    // the first million or so bookmarks
    this.call ('keyLink.getKeyLinks', ['NORMAL', this.mFlockUserType, 1, 
            1000000], listener);
}/*}}}*/
bokeeWebService.prototype.getSyncRequired = function (aListener) {/*{{{*/
    dump ('bokeeWebService.getSyncRequired()\n');
    aListener.onSucceeded (this.mInitialized);
}/*}}}*/
bokeeWebService.prototype.login = function (aUserName, aPassword, aListener) {/*{{{*/
    dump ('bokeeWebService.login("'+aUserName+'", "'+aPassword+'", '+aListener+')\n');
    PREFS.setCharPref (PREF_USERNAME, aUserName);
    PREFS.setCharPref (PREF_PASSWORD, aPassword);

    aListener.failed ('Not Supported');

    // FIXME: check authentication
}/*}}}*/
// }}}

function loadLibrary(aFilename) { // {{{
    var file = Components.classes["@mozilla.org/file/directory_service;1"]
        .getService(Components.interfaces.nsIProperties)
        .get("ComsD", Components.interfaces.nsIFile);
    file.append(aFilename);

    var ios = Components.classes["@mozilla.org/network/io-service;1"]
        .getService(Components.interfaces.nsIIOService);
    var fileHandler = ios.getProtocolHandler("file")
        .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
    var spec = fileHandler.getURLSpecFromFile(file);

    var scriptLoader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
        .getService(Components.interfaces.mozIJSSubScriptLoader);

    scriptLoader.loadSubScript(spec);
}
// }}}

// load the XPCOM template library
loadLibrary("flockXPCOMTemplate.js.lib");

// load the xmlrpc helper library
loadLibrary ('flockXmlRpcHelper.js.lib');



var module = new xpcomModule ('{aa334f5f-9e62-4907-bf09-0d6bd6840d11}',
        '@flock.com/favorites/service/bokee;1', 
        'Bokee Favorites Web Service', 
        bokeeWebService);
module.setFavoritesWebService ('flock-bokee-web-service');
function NSGetModule(aCompMgr, aFileSpec) {
    return module;
};


dump ('LOADED: flockFavoritesWebServiceBokee.js\n');
