//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

const DEBUG = true; /* set to false to suppress debug messages */

const FLOCK_FEED_SERVICE_CONTRACTID = '@flock.com/feed-service;1';
const FLOCK_FEED_SERVICE_CID        = Components.ID('{d6ed5524-cf20-4948-97fa-d590cb1d38b6}');
const flockIFeedService             = Components.interfaces.flockIFeedService;

const FEED_TEMPLATE_DIR             = "chrome://browser/content/flock/feed/templates";

const FEED_TEMPLATE_SINGLE_FEED     = "single_feed.html";
const FEED_TEMPLATE_SINGLE_BY_TOPIC = "single_feed_by_topic.html";
const FEED_TEMPLATE_AGG_BY_FEED     = "agg_by_feed.html";
const FEED_TEMPLATE_AGG_BY_DATE     = "agg_by_date.html";
const FEED_TEMPLATE_AGG_BY_TOPIC    = "agg_by_topic.html";
const FEED_TEMPLATE_ERROR           = "error.html";

const POLL_INTERVAL                 = 1000 * 60 * 60;
//const POLL_INTERVAL                 = 1000 * 15;

const FEED_RDF_NS                   = "http://flock.com/rdf#"

const FEED_DATA_DIR                 = "feeds";

const FEED_DATA_FILENAME            = "feed.rdf";
const FEED_CACHE_FILENAME           = "feed.xml";
const FEED_IMAGE_DIR                = "images";

const FEED_DATA_RDF_PREFIX          = "urn:flock:feed:";

const FEED_DATA_RDF_FEED_ROOT       = FEED_DATA_RDF_PREFIX + "feeds";
const FEED_DATA_RDF_DATE_ROOT       = FEED_DATA_RDF_PREFIX + "dates";

const MAX_POSTS_PER_PAGE            = 200;

const nsISupports                   = Components.interfaces.nsISupports;
const nsIClassInfo                  = Components.interfaces.nsIClassInfo;
const nsITimer                      = Components.interfaces.nsITimer;
const nsITimerCallback              = Components.interfaces.nsITimerCallback;
const nsIProperties                 = Components.interfaces.nsIProperties;
const nsIFile                       = Components.interfaces.nsIFile;
const nsIIOService                  = Components.interfaces.nsIIOService;
const nsIFileProtocolHandler        = Components.interfaces.nsIFileProtocolHandler;
const nsIXMLHttpRequest             = Components.interfaces.nsIXMLHttpRequest;
const nsIDOMParser                  = Components.interfaces.nsIDOMParser;
const nsIDOMSerializer              = Components.interfaces.nsIDOMSerializer;
const nsIRDFResource                = Components.interfaces.nsIRDFResource;
const flockIFavoritesService        = Components.interfaces.flockIFavoritesService;

const TIMER_CONTRACTID              = '@mozilla.org/timer;1';
const DIRECTORY_SERVICE_CONTRACTID  = '@mozilla.org/file/directory_service;1';
const IO_SERVICE_CONTRACTID         = '@mozilla.org/network/io-service;1';
const XMLHTTPREQUEST_CONTRACTID     = '@mozilla.org/xmlextras/xmlhttprequest;1';
const DOMPARSER_CONTRACTID          = '@mozilla.org/xmlextras/domparser;1';
const DOMSERIALIZER_CONTRACTID      = '@mozilla.org/xmlextras/xmlserializer;1';
const FAVORITES_CONTRACTID          = '@mozilla.org/rdf/datasource;1?name=flock-favorites';

const flockIHTML2XHTML              = Components.interfaces.flockIHTML2XHTML;

const HTML2XHTML_CONTRACTID         = "@flock.com/html2xhtml;1";

/* from nspr's prio.h */
const PR_RDONLY                     = 0x01;
const PR_WRONLY                     = 0x02;
const PR_RDWR                       = 0x04;
const PR_CREATE_FILE                = 0x08;

// Clustering configuration (for sorting by topic).
const USE_STEMMER = false;      // stem words?
const USE_STOPWORDS = true;     // eliminate common words?
const USE_BIGRAMS = true;       // use bigrams?
const BOOSTLINKS = 7;           // the amount to boost links by (XXX what
            // about links to www.google.com or similar irrelevant stuff?)
const BOOSTTITLES = 5;          // the amount to boost title words by

// Service implementation
function flockFeedService()
{
    debug("Constructed");

    this.init() 

    this.timer = Components.classes[TIMER_CONTRACTID].getService(nsITimer);
    this.timer.initWithCallback(this, POLL_INTERVAL,
                                nsITimer.TYPE_REPEATING_SLACK);
}

function fixFeedColonURL(url)
{
    if (url.substr(0, 'feed:'.length) == 'feed:') {
        var tmpurl = url.substring('feed:'.length);
        if (tmpurl.substr(0, '//'.length) == '//')
            tmpurl = 'http:' + tmpurl;
        url = tmpurl;
    }
    return url;
}

flockFeedService.prototype.notify =
function (obj)
{
    this.refreshAllFeeds(null);
}

flockFeedService.prototype.test =
function ()
{
    debug("testing!");

    //this.refreshAllFeeds(null);
    var a = [];
    a[0] = 'http://www.schneier.com/blog/atom.xml';
    //a[1] = 'http://xml.metafilter.com/atom.xml';

    //this.refreshFeed(a[0]);
    //this.refreshFeed(a[1]);
    //var ds = this.getAggregateFeeds('dates', a.length, a);
    //return ds;

    //var f = this.getAggregateFile('dates', a.length, a);
    //return f;

    return 'poop';
}

flockFeedService.prototype.init =
function ()
{
    var feedDir = Components.classes[DIRECTORY_SERVICE_CONTRACTID]
        .getService(nsIProperties).get('ProfD', nsIFile);
   
    feedDir.append(FEED_DATA_DIR); 

    if (!feedDir.exists())
        feedDir.createUnique(nsIFile.DIRECTORY_TYPE, 0755);
}

flockFeedService.prototype.hasFeed =
function (url)
{
    var feedFile = getFeedCacheFile(url);
    var curTime = new Date();
    var hasFeed = (feedFile.exists() &&
                   feedFile.lastModifiedTime > curTime - POLL_INTERVAL);
    debug('hasFeed ' + url + ': ' + hasFeed);
    return hasFeed;
}

flockFeedService.prototype.getFeed =
function (url)
{
    return getFeedInternal(fixFeedColonURL(url));
}

flockFeedService.prototype.getFeedData =
function (url, count)
{
    var postlist = [];

    url = fixFeedColonURL(url);
    var feed = this.getFeed(url);

    if (!feed) {
        debug("Error: " + url + " doesn't have a feed!");
        return null;
    }

    var ds = new RDFDataSource();
    ds.Init(feed);

    var root = ds.getNode(FEED_DATA_RDF_PREFIX + url);
    var posts = root.getChildren();
    while (posts && posts.hasMoreElements()) {
        var post = posts.getNext();
        var postinfo = {};

        getFeedNodeTargets(post, postinfo,
                           ['title', 'link', 'datevalue', 'indexvalue']);
        postinfo.datevalue = Number(postinfo.datevalue);

        postlist.push(postinfo);
    }

    var sorter = function(a, b) {
        var res = b.datevalue - a.datevalue;
        if (res == 0) return a.indexvalue - b.indexvalue;
        else return res;
    }

    postlist.sort(sorter);
    if (postlist.length > count.value) {
        postlist.splice(count.value);
    }

    count.value = postlist.length;
    return postlist;
}

flockFeedService.prototype.refreshFeed =
function (url)
{
    var req = Components.classes[XMLHTTPREQUEST_CONTRACTID]
        .createInstance(nsIXMLHttpRequest);

    url = fixFeedColonURL(url);

    try {
        req.open('GET', url, true);
    }
    catch(e) {
        // likely a url error;
        debug("Error loading " + url);
        return;
    }

    req.onreadystatechange = function (aEvt) {
        if (req.readyState == 4) {
            if (req.status == 200)
                TryParseFeed(url, req.responseXML);
            else
                debug("Error loading " + url);
        }
    };

    req.send(null); 
}

flockFeedService.prototype.refreshAllFeeds =
function (listener)
{
    debug('refreshing feeds');
    var fsvc = Components.classes[FAVORITES_CONTRACTID]
                         .getService(flockIFavoritesService);
    var urls = fsvc.getAllFeeds();
    var urlsArr = [];
    while (urls.hasMoreElements()) {
        var urlNode = urls.getNext().QueryInterface(nsIRDFResource);
        urlsArr.push(urlNode.ValueUTF8);
    }

    if (urlsArr.length > 0)
        refreshFeeds(urlsArr, 0, urlsArr.length, listener);
    else
        debug('none to refresh');
}

flockFeedService.prototype.isFeed =
function (dom)
{
    return this.getFeedType(dom) ? true : false;
}

flockFeedService.prototype.getFeedType =
function (dom)
{
    return SniffFeedType(dom);
}

flockFeedService.prototype.loadFeedView =
function (spec, channel)
{
    var urls = [];
    var sortkey = null;
    var page = 1;

    if (spec.substr(0, 'feed:'.length) == 'feed:') {
        urls.push(fixFeedColonURL(spec));
    } else if (spec.substr(0, 'feeds:'.length) == 'feeds:') {
        var query = spec.substring('feeds:'.length);
        urls = query.split('&');

        if (urls.length) {
            for (var i = 0; i < urls.length; i++) {
                var splice = false;
                if (urls[i].substr(0, 'sort='.length) == 'sort=') {
                    sortkey = urls[i].substring('sort='.length);
                    splice = true;
                } else if (urls[i].substr(0, 'page='.length) == 'page=') {
                    page = Number(urls[i].substring('page='.length));
                    splice = true;
                }

                if (splice) {
                    urls.splice(i, 1);
                    i--;
                } else {
                    urls[i] = unescape(urls[i]);
                }
            }
        }
    }

    switch (urls.length) {
        case 0:
            this.loadErrorHTML(channel);
            break;
        case 1:
            if (sortkey == 'topics') this.loadTopicsHTML(urls[0], channel,
                                                         page);
            else this.loadFeedHTML(urls[0], channel, page);
            break;
        default:
            this.loadAggregateHTML(sortkey, urls, channel, page);
            break;
    }
}

flockFeedService.prototype.prioritizeFeeds =
function (len, urls, listener)
{
    getPriorityFeeds(urls, 0, donePriorityFeeds, listener);
}

flockFeedService.prototype.prioritizeFeedsQuick =
function (len, feeds, count)
{
    var info = [];

    var priorities = {
        "application/atom+xml": 100,
        "application/rss+xml": 50,
        "text/xml": 40
    };

    var title_priorities = {
       "Atom 1.0": 100,
       "Atom 0.3": 75,
       "RSS 2.0": 50,
       "RSS 0.92": 40
    };

    function sorter(a,b) {
        var t1 = priorities[a.type];
        var t2 = priorities[b.type];

        if (!t1) t1 = title_priorities[a.title];
        if (!t2) t2 = title_priorities[b.title];

        if (!t1) t1 = 0;
        if (!t2) t2 = 0;

        return t2 - t1;
    }

    feeds.sort(sorter);

    // XXX: could be smarter
    count.value = 1;
    return [feeds[0]];
}

// the nsIClassInfo implementation
flockFeedService.prototype.flags = nsIClassInfo.Singleton;
flockFeedService.prototype.classDescription = "Flock Feed Service";
flockFeedService.prototype.getInterfaces =
function (count)
{
    var interfaceList = [flockIFeedService, nsIClassInfo, nsITimerCallback];
    count.value = interfaceList.length;
    return interfaceList;
}

flockFeedService.prototype.getHelperForLanguage =
function (language)
{
    return null;
}

// the nsISupports implementation
flockFeedService.prototype.QueryInterface =
function (iid)
{
    if (!iid.equals(flockIFeedService) && 
	!iid.equals(nsITimerCallback) &&
	!iid.equals(nsIClassInfo) &&
        !iid.equals(nsISupports))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
}

// Module implementation
var feedModule = new Object();

feedModule.registerSelf =
function (compMgr, fileSpec, location, type)
{
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    compMgr.registerFactoryLocation(FLOCK_FEED_SERVICE_CID, 
                                    "Flock Feeds JS Component",
                                    FLOCK_FEED_SERVICE_CONTRACTID, 
                                    fileSpec, 
                                    location,
                                    type);
}

feedModule.getClassObject =
function (compMgr, cid, iid)
{
    if (!cid.equals(FLOCK_FEED_SERVICE_CID))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    return feedServiceFactory;
}

feedModule.canUnload =
function(compMgr)
{
    debug("Unloading component.");
    return true;
}
    
/* factory object */
var feedServiceFactory = new Object();

feedServiceFactory.createInstance =
function (outer, iid)
{
    debug("CI: " + iid);
    if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

    return (new flockFeedService()).QueryInterface(iid);
}

/* entrypoint */
function NSGetModule(compMgr, fileSpec)
{
    return feedModule;
}

if (DEBUG) {
    debug = function (s) { dump("-*- flockFeedService: " + s + "\n"); }
} else {
    debug = function (s) {}
}

function getStreamData(istream)
{
    var sstream = Components.classes['@mozilla.org/scriptableinputstream;1'].
        createInstance(Components.interfaces.nsIScriptableInputStream);

    sstream.init(istream); 

    var data = '';
    var str = sstream.read(-1);
    while (str.length > 0) {
        data += str;
        str = sstream.read(-1);
    }

    sstream.close();
    istream.close();

    return data;
}

function getTemplate(filename)
{
    var url = FEED_TEMPLATE_DIR + '/' + filename;

    var ios = Components.classes[IO_SERVICE_CONTRACTID]
        .createInstance(nsIIOService);

    var channel = ios.newChannel(url, null, null);
    var cstream = channel.open();

    return getStreamData(cstream);
}

function escape_url(url)
{
    var rval = url.replace(/\/+$/, '');
    rval = rval.replace(/[\?\/\\:.]+/g, '_');
    return escape(rval);
}

function getFeedDir(url)
{
    var feedDir = Components.classes[DIRECTORY_SERVICE_CONTRACTID]
        .getService(nsIProperties).get('ProfD', nsIFile);

    feedDir.append(FEED_DATA_DIR);
    feedDir.append(escape_url(url));

    return feedDir;
}

function getFeedFile(url)
{
    var feedFile = getFeedDir(url);
    feedFile.append(FEED_DATA_FILENAME);
    return feedFile;
}

function getFeedCacheFile(url)
{
    var feedFile = getFeedDir(url);
    feedFile.append(FEED_CACHE_FILENAME);
    return feedFile;
}

function getFeedImageDir(url)
{
    var feedFile = getFeedDir(url);
    feedFile.append(FEED_IMAGE_DIR);
    return feedFile;
}

function getFeedInternal(url)
{
    var feedFile = getFeedFile(url);

    if (!feedFile.exists())
        return null;

    var ios = Components.classes[IO_SERVICE_CONTRACTID]
        .getService(nsIIOService);

    var fileHandler = ios.getProtocolHandler('file')
        .QueryInterface(nsIFileProtocolHandler);

    var spec = fileHandler.getURLSpecFromFile(feedFile);

    var ds = new RDFDataSource(spec);

    return ds.getRawDataSource();
}

function saveData(file, xmlDocument)
{
    if (file.exists())
        file.remove(false);

    file.createUnique(nsIFile.NORMAL_FILE_TYPE, 0644);

    var ostream = Components.classes['@mozilla.org/network/file-output-stream;1']
        .createInstance(Components.interfaces.nsIFileOutputStream);

    ostream.init(file, PR_WRONLY | PR_CREATE_FILE, 0644, 0);
    
    var serializer = Components.classes["@mozilla.org/xmlextras/xmlserializer;1"]
        .getService(Components.interfaces.nsIDOMSerializer);
        
    serializer.serializeToStream(xmlDocument, ostream, "UTF-8");
    
    ostream.flush();
    ostream.close();
}

function SniffFeedType(dom)
{
    var nodes = dom.getElementsByTagName('feed');
    if (nodes.length > 0) {
        var node = nodes[0];
        var ns = node.getAttribute('xmlns');

        if (ns == 'http://www.w3.org/2005/Atom')
            return 'Atom 1.0';
        else
            return 'Atom 0.3';
    }

    nodes = dom.getElementsByTagName('RDF');
    if (nodes.length > 0) {
        var node = nodes[0];

        if (dom.getElementsByTagName('channel').length == 0)
            return null;

        var ns = node.getAttribute('xmlns');
        if (ns == 'http://purl.org/rss/1.0/')
            return 'RSS 1.0';
        else
            return 'RSS 0.9';
    }

    nodes = dom.getElementsByTagName('rss');
    if (nodes.length > 0) {
        var node = nodes[0];

        if (dom.getElementsByTagName('channel').length == 0)
            return null;

        var version = node.getAttribute('version');

        if (version)
            return 'RSS ' + version;
        else
            return 'RSS';
    }

    return null;
}

function TryParseFeed(url, xmlDocument)
{
    debug("Parsing " + url);

    var cacheFile = getFeedCacheFile(url);
    saveData(cacheFile, xmlDocument);
 
    var feedFile = getFeedFile(url);

    if (!feedFile.exists())
        feedFile.createUnique(nsIFile.NORMAL_FILE_TYPE, 0644);

    var ios = Components.classes[IO_SERVICE_CONTRACTID]
        .getService(nsIIOService);

    var fileHandler = ios.getProtocolHandler('file')
        .QueryInterface(nsIFileProtocolHandler);

    var spec = fileHandler.getURLSpecFromFile(feedFile);

    var ds = new RDFDataSource(spec);

    var dateroot = ds.getNode(FEED_DATA_RDF_DATE_ROOT);
    dateroot.makeSeq();

    var feedroot = ds.getNode(FEED_DATA_RDF_FEED_ROOT);
    feedroot.makeBag();

    var root = ds.getNode(FEED_DATA_RDF_PREFIX + url);
    root.makeBag();

    feedroot.addChild(root);

    var parser = Components.classes[DOMPARSER_CONTRACTID]
        .createInstance(nsIDOMParser);
    var isAtom = false;

    var dom = xmlDocument;

    var feedtype = SniffFeedType(dom);
    if (!feedtype)
        feedtype = "Unknown";

    root.addTargetOnce(FEED_RDF_NS + "feedtype", feedtype);
    
    var feeds = dom.getElementsByTagName('feed');

    var channel = null;
    if (feeds.length > 0) {
        channel = feeds[0];
        isAtom = true;
    } else {
        var channels = dom.getElementsByTagName('channel');
        if (channels.length > 0) {
            channel = channels[0];
        }
    }

    if (channel == null) {
        debug("Error parsing " + url);
        feedFile.remove(false);
        return;
    }

    var feed = {};

    var currDate = new Date();
    feed.date = currDate.toString();

    for (var i = 0; i < channel.childNodes.length; i++) {
        var n = channel.childNodes[i];
        if (n.nodeType == Components.interfaces.nsIDOMNode.ELEMENT_NODE) {
            switch (n.nodeName) {
                case "title":
                case "subtitle":
                case "link":
                case "description":
                case "tagline":
                case "dc:date":
                case "pubDate":
                case "author":
                //case "ttl":
                //case "id":
                    var name = n.nodeName;
                    var node_data;

                    if (name == "tagline" || name == "subtitle")
                        name = "description";
                    else if (name == "managingEditor")
                        name = "author";

                    if (name == "description")
                        node_data = getHTMLContent(n);
                    else if (isAtom && name == "author")
                        node_data = getAuthor(n);
                    else if (name == "link")
                        node_data = getLinkURL(n);
                    else
                        node_data = getNodeText(n);

                    if (name == "dc:date" || name == "pubDate" ||
                        name == "updated" || name == "modified") {
                        name = "date";
                        node_data = parseDate(node_data);
                    }

                    if (node_data)
                        feed[name] = node_data;

                    break;
            }
        }
    }

    if (!feed.title)
        feed.title = url;

    feed.datevalue = getDateValue(new Date(feed.date));
    delete feed.date;

    for (var f in feed) {
        root.addTargetOnce(FEED_RDF_NS + f, feed[f]);
    }
    
    now = new Date();
    root.addTargetOnce(FEED_RDF_NS + "last_refreshed", now.toString());

    var images = dom.getElementsByTagName('image');
    if (images.length > 0) {
        var image = images[0];

        for (var i = 0; i < image.childNodes.length; i++) {
            var c = image.childNodes[i];
            if (c.nodeType == Components.interfaces.nsIDOMNode.ELEMENT_NODE) {
                switch (c.nodeName) {
                    case "url":
                        root.addTargetOnce(FEED_RDF_NS + 'image', getNodeText(c));
                        break;
                }
            }
        }
    }

    var items;
    if (isAtom) {
        items = dom.getElementsByTagName('entry'); 
    } else {
        items = dom.getElementsByTagName('item'); 
    }

    var posts = {};
    var authors = {};
    for (var i = 0; i < items.length; i++) {
        var n = items[i];
        var post = {};

        for (var j = 0; j < n.childNodes.length; j++) {
            var c = n.childNodes[j];
            if (c.nodeType == Components.interfaces.nsIDOMNode.ELEMENT_NODE) {
                switch (c.nodeName) {
                    case "title":
                    case "author":
                    case "link":
                    case "description":
                    case "content:encoded":
                    case "content":
                    case "summary":
                    case "dc:date":
                    case "pubDate":
                    case "modified":
                    case "updated":
                    case "issued":
                    case "published":
                    case "guid":
                    case "id":
                        var name = c.nodeName;
                        var node_data;

                        if (name == 'content:encoded')
                            name = "content";

                        if (name == "content" || name == "description") {
                            if (isAtom)
                                node_data = getAtomContent(c);
                            else
                                node_data = getHTMLContent(c);
                        } else if (isAtom && name == "author")
                            node_data = getAuthor(c);
                        else if (name == "link")
                            node_data = getLinkURL(c);
                        else
                            node_data = getNodeText(c);

                        if (name == "dc:date" || name == "pubDate" ||
                            name == "issued" || name == "published") {
                            name = "date";
                            node_data = parseDate(node_data);
                        } else if (name == "updated" || name == "modified") {
                            name = "updated";
                            node_data = parseDate(node_data);
                        }

                        if (node_data)
                            post[name] = node_data;

                        break;
                }
            }
        }

        var post_id = null;
        if (post.id)
            post_id = post.id;
        else if (post.guid)
            post_id = post.guid;
        else if (post.link)
            post_id = post.link;
        else if (post.date)
            post_id = post.date;
        else if (post.title)
            post_id = post.title;

        var post_desc = null;
        if (post.content)
            post_desc = post.content;
        else if (post.description)
            post_desc = post.description;
        else if (post.summary)
            post_desc = post.summary;
        else if (post.link)
            post_desc = post.link;

        if (!post.date && post.updated)
            post.date = post.updated;

        //post.description = post_desc + "\n";
        post.description = post_desc;
      
        if (feed.link) 
            post.feedurl = feed.link;

        post.feedtitle = feed.title;

        var date;
        if (post.date) {
            date = new Date(post.date);
        } else {
            date = getDateFromValue(feed.datevalue);
        }

        if (post.author)
            authors[post.author] = 1;
        else
            authors[''] = 1;

        post.datevalue = getDateValue(date);

        var date_only = new Date(date.toDateString());

        post.indexvalue = String(i);

        var pnode;
        if (post_id)
            pnode = ds.getNode(FEED_DATA_RDF_PREFIX + "post:" + post_id);
        else
            pnode = ds.getAnonymousNode();

        for (var p in post) {
            pnode.addTargetOnce(FEED_RDF_NS + p, post[p]);
        }

        root.addChild(pnode);

        var dateNode = ds.getNode(FEED_DATA_RDF_DATE_ROOT + ':' + date_only.getTime());

        dateNode.makeBag();
        dateNode.addChild(pnode);

        dateroot.addChild(dateNode);
    }

    if (isAtom && !feed.author) {
        var authorCount = 0;
        var author1;
        for (var a in authors) {
            authorCount++;
            if (authorCount > 1)
                break;
            author1 = a;
        }

        if (authorCount == 1 && author1) {
            root.addTargetOnce(FEED_RDF_NS + "author", author1);
        }
    }
}

function getNodeText(node)
{
    var text = "";
    for (var i = 0; i < node.childNodes.length; i++) {
        var n = node.childNodes[i];
        if (n.nodeType == Components.interfaces.nsIDOMNode.TEXT_NODE ||
            n.nodeType == Components.interfaces.nsIDOMNode.CDATA_SECTION_NODE) {
            text += n.nodeValue;
        }
    }
    return text;
} 

function getLinkURL(node)
{
    if (node.hasAttribute("href")) {
        if (!node.hasAttribute("rel") || node.getAttribute("rel") == "alternate")
            return node.getAttribute("href");
    } else
        return getNodeText(node);
}

function getHTMLContent(node)
{
    return getNodeText(node);

    var html2xhtml = Components.classes[HTML2XHTML_CONTRACTID]
        .createInstance(flockIHTML2XHTML);

    return html2xhtml.convert(getNodeText(node));
}

function getAuthor(node)
{
    for (var i = 0; i < node.childNodes.length; i++) {
        var n = node.childNodes[i];
        if (n.nodeType == Components.interfaces.nsIDOMNode.ELEMENT_NODE) {
            if (n.nodeName == "name")
                return getNodeText(n);
        }
    }

    return null;
}

function getAtomContent(node)
{
    if (node.hasAttribute("type") &&
        (node.getAttribute("type") == "xhtml" ||
         node.getAttribute("type") == "application/xhtml+xml")) {
        var serializer = Components.classes[DOMSERIALIZER_CONTRACTID]
            .createInstance(nsIDOMSerializer);

        for (var i = 0; i < node.childNodes.length; i++) {
            var n = node.childNodes[i];
            if (n.nodeType == Components.interfaces.nsIDOMNode.ELEMENT_NODE) {
                return serializer.serializeToString(n);
            }
        }
    }
    else if (node.hasAttribute("type") &&
             (node.getAttribute("type") == "html" ||
              node.getAttribute("type") == "text/html")) {
        return getHTMLContent(node);
    }

    return getNodeText(node);
}

function refreshFeeds(urls, current, total, listener)
{
    if (current == total) {
        debug("done refreshing feeds");
        if (listener)
            listener.onProgress(current, total, 'Done');
        return;
    }

    var url = fixFeedColonURL(urls[current]);

    if (listener)
        listener.onProgress(current, total, 'Loading ' + url);

    var req = Components.classes[XMLHTTPREQUEST_CONTRACTID]
        .createInstance(nsIXMLHttpRequest);

    try {
        req.open('GET', url, true);
    }
    catch(e) {
        // likely a url error;
        debug("Error loading " + url);
        refreshFeeds(urls, current + 1, total, listener);
        return;
    }

    req.onreadystatechange = function (aEvt) {
        if (req.readyState == 4) {
            if (req.status == 200)
                TryParseFeed(url, req.responseXML);
            else
                debug("Error loading " + url);

            refreshFeeds(urls, current + 1, total, listener);
        }
    };

    try {
        req.send(null);
    }
    catch(e) {
        // likely a url error;
        debug("Error sending to " + url);
        refreshFeeds(urls, current + 1, total, listener);
        return;
    }
}

function getFeedNodeTargets(node, store, targets)
{
    for (var i = 0; i < targets.length; i++) {
        var target = targets[i];
        var val = node.getTarget(FEED_RDF_NS + target);
        if (val) { 
            store[target] = val.getValue();
        }
    }
}

flockFeedService.prototype.getTData =
function (urls, page)
{
    var tdata = { feeds : {}, page : page };
    var postlist = [];

    for (i = 0; i < urls.length; i++) {
        var url = urls[i];
        if (url.substr(0, 'http://'.length) == 'http://')
            var singleurl = 'feed' + url.substring('http'.length);
        else
            var singleurl = 'feed:' + url;

        var feed_rawds = this.getFeed(url);
        if (!feed_rawds) continue;

        var feed_ds = new RDFDataSource();
        feed_ds.Init(feed_rawds);

        var feed = feed_ds.getNode(FEED_DATA_RDF_PREFIX + url);
        var feedinfo = { srcurl: url, singleurl: singleurl };
        getFeedNodeTargets(feed, feedinfo,
                           ['link', 'title', 'subtitle', 'author']);
        feedinfo['url'] = url;
        tdata.feeds[url] = feedinfo;

        var root = feed_ds.getNode(FEED_DATA_RDF_PREFIX + url);
        var posts = root.getChildren();
        while (posts && posts.hasMoreElements()) { 
            var post = posts.getNext();
            var postinfo = {};
            getFeedNodeTargets(post, postinfo,
                               ['title', 'datevalue', 'indexvalue', 'author',
                                'link', 'description']);
            postinfo.datevalue = Number(postinfo.datevalue);
            postinfo.indexvalue = Number(postinfo.indexvalue);
            postinfo.feed = url;
            postlist.push(postinfo);
        }
    }
    var sorter = function(a, b) {
        var res = b.datevalue - a.datevalue;
        if (res == 0) return a.indexvalue - b.indexvalue;
        else return res;
    };
    postlist.sort(sorter);
    tdata.totalPosts = postlist.length;
    tdata.allPosts = postlist.splice(MAX_POSTS_PER_PAGE * (page - 1),
                                     MAX_POSTS_PER_PAGE);
    makeSortURLs(tdata, urls);
    return tdata;
}


flockFeedService.prototype.aggByDate =
function (urls, page)
{
    var store = {};

    var tdata = this.getTData(urls, page);
    tdata.dates = [];
    tdata.posts = {};

    for (var i = 0; i < tdata.allPosts.length; i++) {
        var post = tdata.allPosts[i];
        var date_only = new Date(new Date(post.datevalue).toDateString());
        var dateval = date_only.getTime();
        if (tdata.dates.length == 0 ||
            tdata.dates[tdata.dates.length - 1] != dateval) {
            tdata.dates.push(dateval);
            tdata.posts[dateval] = [];
        }
        tdata.posts[dateval].push(post);
    }

    return tdata;
}

flockFeedService.prototype.aggDates =
function (urls, page)
{
    var tdata = this.aggByDate(urls, page);
    tdata.viewtitle = 'Aggregated Feeds';

    var template = getTemplate(FEED_TEMPLATE_AGG_BY_DATE);
    return template.process(tdata);
}

flockFeedService.prototype.aggFeeds =
function (urls, page)
{
    var tdata = this.getTData(urls, page);
    tdata.viewtitle = 'Aggregated Feeds';

    for (var feed in tdata.feeds) {
        tdata.feeds[feed].posts = [];
    }
    for (var i = 0; i < tdata.allPosts.length; i++) {
        tdata.feeds[tdata.allPosts[i].feed].posts.push(tdata.allPosts[i]);
    }

    var template = getTemplate(FEED_TEMPLATE_AGG_BY_FEED);
    return template.process(tdata);
}

flockFeedService.prototype.aggByTopic =
function (urls, page)
{
    var tdata = this.getTData(urls, page);

    for (var i = 0; i < tdata.allPosts.length; i++) {
        var table = getWords(tdata.allPosts[i]);
        tdata.allPosts[i].words = '';
        for (var str in table) {
            tdata.allPosts[i].words += str + '|' + table[str] +'\n';
        }
    }
    var clusterer = Components.classes['@flock.com/clusterer;1']
        .getService(Components.interfaces.flockIClusterer);
    var clusters = clusterer.cluster(tdata.allPosts.length, tdata.allPosts);
    tdata.topics = [];
    tdata.other = []
    for (var i = 0; i < clusters.length; i++) {
        var cluster = clusters.queryElementAt(i, Components.interfaces
                                                           .flockICluster);
        if (cluster.posts.length == 1) {
            tdata.other.push(tdata.allPosts[cluster.posts
                .queryElementAt(0, Components.interfaces.nsISupportsPRUint32)
                .data]);
            continue;
        }

        var posts = new Array(cluster.posts.length);
        for (var j = 0; j < cluster.posts.length; j++) {
            posts[j] = tdata.allPosts[cluster.posts
                .queryElementAt(j, Components.interfaces.nsISupportsPRUint32)
                .data];
        }
        tdata.topics.push({
            srcurl : "cluster" + i,
            word : cluster.keyword,
            word2 : cluster.keyword2,
            posts : posts
        });
    }

    var date_sorter = function(a, b) {
        var res = b.datevalue - a.datevalue;
        if (res == 0) return a.indexvalue - b.indexvalue;
        else return res;
    };
    var cluster_sorter = function(a, b) {
        if (a.posts.length == b.posts.length)
            return date_sorter(a.posts[0], b.posts[0]);
        return b.posts.length - a.posts.length;
    };
    tdata.topics.sort(cluster_sorter);
    tdata.other.sort(date_sorter);
    for (var i = 0; i < tdata.topics.length; i++) {
        tdata.topics[i].posts.sort(date_sorter);
    }

    return tdata;
}

flockFeedService.prototype.loadTopicsHTML =
function (url, channel, page)
{
    debug("loadTopicsHTML: " + url);

    if (channel.bypassCache || !this.hasFeed(url)) {
        channel.bypassCache = false; // so we don't bypass infinitely

        var req = Components.classes[XMLHTTPREQUEST_CONTRACTID]
            .createInstance(nsIXMLHttpRequest);

        try {
            req.open('GET', url, true);
        }
        catch(e) {
            // likely a url error;
            this.loadErrorHTML(channel);
            return;
        }

        var feedsrv = this;

        req.onreadystatechange = function (aEvt) {
            if (req.readyState == 4) {
                if (req.status == 200) {
                    TryParseFeed(url, req.responseXML);
                    feedsrv.loadTopicsHTML(url, channel, page);
                }
                else
                    feedsrv.loadErrorHTML(channel);
            } 
        };

        req.send(null);
        return;
    }
    var urls = [url];
    var tdata = this.aggByTopic(urls, page);

    tdata.feed = tdata.feeds[url];

    var template = getTemplate(FEED_TEMPLATE_SINGLE_BY_TOPIC);
    sendData(channel, template.process(tdata));
}

flockFeedService.prototype.aggTopics =
function (urls, page)
{
    var tdata = this.aggByTopic(urls, page);
    tdata.viewtitle = 'Aggregated Feeds';

    var template = getTemplate(FEED_TEMPLATE_AGG_BY_TOPIC);
    return template.process(tdata);
}

function cleanWords(words) {
    // Takes in a string, cleans it up a bit, and returns an array of the
    // words in it.
    // Throws out all punctuation and escaped characters (such as &nbsp;)
    // except for apostrophes, dashes, and periods that occur in the middle of
    // a word.
    return words.replace(/&((#8217)|(#39)|(apos)|(rsquo));/g, "'")
                .replace(/&((ndash)|(#8211));/g, '-')
                .replace(/&[^; ]*;/g, ' ') // get rid of other html entities
                .replace(/[^a-zA-Z0-9'.-]/g, ' ') // other punctuation
                .replace(/(['.-]\s)|(\s['.-])|(^['.-])|(['.-]$)/g, ' ')
                .split(/\s+/);
}
function addWord(word, tf, table, stem, stop)
{
    if (tf == 0) return;
    if (stop && STOPWORDS.hasOwnProperty(word)) return;
    if (stem) word = stemWord(word);
    if (table.hasOwnProperty(word)) {
        table[word] += tf;
    } else {
        table[word] = tf;
    }
}
function getWords(postinfo) {
    var table = {};
    var desc = postinfo.description.toLowerCase();
    var words = cleanWords(desc.replace(/<[^>]*>/g,' '));
    for (var i = 0; i < words.length; i++) {
        addWord(words[i], 1, table, USE_STEMMER, USE_STOPWORDS);
    }

    if (USE_BIGRAMS) {
        for (var i = 0; i < words.length-1; i++) {
            // XXX: might want to stem words[i] and words[i+1], not the same
            // as USE_STEMMER though.  Could also give bigrams a different TF
            // weight.
            if (USE_STOPWORDS &&
                (STOPWORDS.hasOwnProperty(words[i]) ||
                 STOPWORDS.hasOwnProperty(words[i+1]))) continue;
            addWord(words[i] + ' ' + words[i+1], 1, table,
                    false, false); // stem/stop doesn't make sense for bigrams
        }
    }
    var links = desc.match(/http[^'" \t\n]+/);
    if (links != null) {
        for (var i = 0; i < links.length; i++) {
            addWord(links[i], BOOSTLINKS, table, false, false);
        }
    }
    if (postinfo.title != null) {
        words = cleanWords(postinfo.title.toLowerCase());
        for (var i = 0; i < words.length; i++) {
            addWord(words[i], BOOSTTITLES, table, USE_STEMMER, USE_STOPWORDS);
        }
        if (USE_BIGRAMS) {
            for (var i = 0; i < words.length-1; i++) {
                if (USE_STOPWORDS &&
                    (STOPWORDS.hasOwnProperty(words[i]) ||
                     STOPWORDS.hasOwnProperty(words[i+1]))) continue;
                addWord(words[i] + ' ' + words[i+1], BOOSTTITLES, table, false,
                        USE_STOPWORDS);
            }
        }
    }
    if (postinfo.link != null) {
        addWord(postinfo.link.toLowerCase(), BOOSTLINKS, table, false, false);
    }

    return table;
}

flockFeedService.prototype.loadFeedHTML =
function (url, channel, page)
{
    debug("loadFeedHTML: " + url);

    if (channel.bypassCache || !this.hasFeed(url)) {
        channel.bypassCache = false; // so we don't bypass infinitely

        var req = Components.classes[XMLHTTPREQUEST_CONTRACTID]
            .createInstance(nsIXMLHttpRequest);

        try {
            req.open('GET', url, true);
        }
        catch(e) {
            // likely a url error;
            this.loadErrorHTML(channel);
            return;
        }

        var feedsrv = this;

        req.onreadystatechange = function (aEvt) {
            if (req.readyState == 4) {
                if (req.status == 200) {
                    TryParseFeed(url, req.responseXML);
                    feedsrv.loadFeedHTML(url, channel, page);
                }
                else
                    feedsrv.loadErrorHTML(channel);
            } 
        };

        req.send(null);
        return;
    }

    var urls = [url];
    var tdata = this.aggByDate(urls, page);

    tdata.feed = tdata.feeds[url];
 
    var template = getTemplate(FEED_TEMPLATE_SINGLE_FEED);
    sendData(channel, template.process(tdata));
}

flockFeedService.prototype.loadFeeds =
function (load_urls, index, sortkey, urls, channel, page)
{
    if (index >= load_urls.length) {
        this.loadAggregateHTML(sortkey, urls, channel, page);
        return;
    }

    var url = load_urls[index];

    var req = Components.classes[XMLHTTPREQUEST_CONTRACTID]
        .createInstance(nsIXMLHttpRequest);

    try {
        req.open('GET', url, true);
    }
    catch(e) {
        // likely a url error;
        debug("Error loading " + url);
        // need to remove this url from the view so we don't keep trying to
        // get it
        load_urls.splice(index, 1);
        for (var i = 0; i < urls.length; i++) {
            if (urls[i] == url) {
                urls.splice(i, 1);
                i--;
            }
        }
        this.loadFeeds(load_urls, index, sortkey, urls, channel, page);
    }

    var feedsrv = this;

    req.onreadystatechange = function (aEvt) {
       if (req.readyState == 4) {
           if (req.status == 200)
               TryParseFeed(url, req.responseXML);
           else {
               debug("Error loading " + url);
               // need to remove this url from the view so we don't keep
               // trying to get it
               load_urls.splice(index, 1);
               for (var i = 0; i < urls.length; i++) {
                   if (urls[i] == url) {
                       urls.splice(i, 1);
                       i--;
                   }
               }
           }

           var oneshot = {
               notify: function (obj) {
                   feedsrv.loadFeeds(load_urls, index + 1, sortkey, urls,
                                     channel, page);
               }
           };

           var timer = Components.classes[TIMER_CONTRACTID]
               .getService(nsITimer);
           timer.initWithCallback(oneshot, 1, nsITimer.TYPE_ONE_SHOT);
       }
    };

    req.send(null);
}

flockFeedService.prototype.loadAggregateHTML =
function (sortkey, urls, channel, page)
{
    debug("loadAggregateHTML: " + sortkey + ", " + urls);
    debug("channel: " + channel);

    var load_urls = [];

    for (var i = 0; i < urls.length; i++) {
        var url = urls[i];

        if (channel.bypassCache || !this.hasFeed(url))
            load_urls.push(url);
    }
    channel.bypassCache = false; // so we don't bypass infinitely

    if (load_urls.length) {
        this.loadFeeds(load_urls, 0, sortkey, urls, channel, page);
        return;
    }
 
    var data = null;
 
    if (sortkey == "feeds") 
        data = this.aggFeeds(urls, page);
    else if (sortkey == "topics")
        data = this.aggTopics(urls, page);
    else
        data = this.aggDates(urls, page);

    sendData(channel, data);
}

flockFeedService.prototype.loadErrorHTML =
function (channel)
{
    var template = getTemplate(FEED_TEMPLATE_ERROR);
    sendData(channel, template);
}

function sendData(channel, data)
{
    var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
        .getService(Components.interfaces.nsIScriptableUnicodeConverter);

    converter.charset = "UTF-8";

    var stream = converter.convertToInputStream(data);

    channel.sendStream(stream, 0, data.length*2);

    channel.finish();
}

function donePriorityFeeds(urls, listener)
{
    var feeds = {};

    for (var u = 0; u < urls.length; u++) {
        var url = fixFeedColonURL(urls[u]);
        var feed = getFeedInternal(url);

        if (!feed) {
            debug("Error: " + url + " doesn't have a feed!");
            continue;
        }

        var ds = new RDFDataSource();
        ds.Init(feed);

        var feedNode = ds.getNode(FEED_DATA_RDF_PREFIX + url);
        var title = feedNode.getTarget(FEED_RDF_NS + 'title').getValue();

        if (!feeds[title]) {
            feeds[title] = new Array();
        }

        var feedinfo = {};
        feedinfo.type = feedNode.getTarget(FEED_RDF_NS + 'feedtype').getValue();
        feedinfo.url = url;

        feeds[title].push(feedinfo);
    }
 
    var priorities = {
       "Atom 1.0": 100,
       "Atom 0.3": 75,
       "RSS 2.0": 50,
       "RSS 0.92": 40
    };

    function sorter(a,b) {
        var t1 = priorities[a.type];
        var t2 = priorities[b.type];

        if (!t1) t1 = 0;
        if (!t2) t2 = 0;

        return t2 - t1;
    }

    var returls = [];

    for (var t in feeds) {
         feeds[t].sort(sorter);
         returls.push(feeds[t][0].url);
    }

    listener.onComplete(returls.length, returls);
}

function getEventQueue()
{
    var eqsrv = Components.classes["@mozilla.org/event-queue-service;1"]
        .getService(Components.interfaces.nsIEventQueueService);
    return eqsrv.getSpecialEventQueue(Components.interfaces.nsIEventQueueService.CURRENT_THREAD_EVENT_QUEUE);
}

function getDateValue(date)
{
   return String(date.getTime());
}

function getDateFromValue(datevalue)
{
    var val = Number(datevalue);
    return new Date(val);
}

/* this is a parser for the W3C subset of ISO 8601 dates
 * http://www.w3.org/TR/NOTE-datetime
 */
const re_iso8601 = new RegExp(
    "^(\\d{4})(-(\\d{2,3})(-(\\d{2})" +
    "(T(\\d{2}):(\\d{2})(:(\\d{2})(\\.(\\d+))?)?" +
    "(Z|(([-+])(\\d{2}):(\\d{2})))?)?)?)?$"
);


function parseDate(datestr)
{
    var date = null;

    datestr = datestr.replace(/^\s*|\s*$/g, '');

    var dateval = Date.parse(datestr);

    if (dateval) {
        date = new Date(dateval);
    } else {
        var m = datestr.match(re_iso8601);

        if (m) {
            date = new Date(Date.UTC(m[1]))

            m[3]  && date.setUTCMonth   (m[3] - 1);
            m[5]  && date.setUTCDate    (m[5]);
            m[7]  && date.setUTCHours   (m[7]);
            m[8]  && date.setUTCMinutes (m[8]);
            m[10] && date.setUTCSeconds (m[10]);

            m[12] && date.setUTCMilliseconds(('0.' + m[12]) * 1000);

            if (m[14]) {
                var tz_off = m[16] * 60 + Number(m[17]);
                tz_off *= m[15] == '-' ? -1 : 1;
                tz_off *= 60 * 1000;

                date.setTime(date.getTime() - tz_off);
            }

        }
    }

    if (date)
        return date.toString();
    else
        return null;
}

function getPriorityFeeds(urls, index, internalComplete, externalComplete)
{
    if (index >= urls.length) {
        internalComplete(urls, externalComplete);
        return;
    }

    var url = fixFeedColonURL(urls[index]);

    var req = Components.classes[XMLHTTPREQUEST_CONTRACTID]
        .createInstance(nsIXMLHttpRequest);

    try {
        req.open('GET', url, true);
    }
    catch(e) {
        // likely a url error;
        debug("Error loading " + url + "\n");
        getPriorityFeeds(urls, index + 1, internalComplete, externalComplete);
        return;
    }

    req.onreadystatechange = function (aEvt) {
        if (req.readyState == 4) {
            if (req.status == 200)
                TryParseFeed(url, req.responseXML);
            else
                debug("Error loading " + url + "\n");

            getPriorityFeeds(urls, index + 1,
                             internalComplete, externalComplete);
        }
    };

    req.send(null);
}

function makeSortURLs(tdata, urls)
{
    var sortkeys = ['dates', 'feeds', 'topics'];

    var escaped_urls = urls.map(function (u) { return escape(u); });

    var query = '&' + escaped_urls.join('&')

    for (var i in sortkeys) {
        sortkey = sortkeys[i];
        tdata['sorturl' + sortkey] = 'feeds:sort=' + sortkey + query;
    }
}

function loadLibraryFromSpec(spec)
{
    var loader = Components.classes['@mozilla.org/moz/jssubscript-loader;1']
        .getService(Components.interfaces.mozIJSSubScriptLoader);
    loader.loadSubScript(spec);
}

loadLibraryFromSpec('chrome://browser/content/flock/contrib/rdfds.js');
loadLibraryFromSpec('chrome://browser/content/flock/contrib/tptemplate.js');

if (USE_STOPWORDS)
    loadLibraryFromSpec('chrome://browser/content/flock/feed/xpinclude/english.stop.js');

if (USE_STEMMER)
    loadLibraryFromSpec('chrome://browser/content/flock/feed/xpinclude/porterstemmer.js');

/*
 * TODO
 *
 * handle ttl, hour stuff
 */

function dd(obj) {
    for (p in obj) {
        try {
            dump(p + ' :: ' + obj[p] + '\n');
            if (typeof obj[p] == 'object') {
                for (q in obj[p]) {
                    dump(p + '.' + q + ' :: ' + obj[p][q] + '\n');
                }
            }
        } catch(e) {
            continue;
        }
    }
}

