//
// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005
// http://flock.com
//
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL
//

const DEBUG = false;         

const FLICKR_PHOTO_CID               = Components.ID('{db720a5c-6315-49bf-a39f-b4d4aa5ed142}');

const nsISupports                   = Components.interfaces.nsISupports;
const nsIClassInfo                  = Components.interfaces.nsIClassInfo;
const nsIFactory                    = Components.interfaces.nsIFactory;
const nsIProperties                 = Components.interfaces.nsIProperties;
const nsILocalFile                  = Components.interfaces.nsILocalFile;
const nsIFile                       = Components.interfaces.nsIFile;
const nsIIOService                  = Components.interfaces.nsIIOService;
const nsIFileProtocolHandler        = Components.interfaces.nsIFileProtocolHandler;
const nsIRDFRemoteDataSource        = Components.interfaces.nsIRDFRemoteDataSource;
const nsIRDFDataSource              = Components.interfaces.nsIRDFDataSource;
const flockIPhotoAPI                = Components.interfaces.flockIPhotoAPI;

const FLICKR_PHOTO_CONTRACTID       = '@flock.com/?photo-api-flickr;1';
const DIRECTORY_SERVICE_CONTRACTID  = '@mozilla.org/file/directory_service;1';
const LOCAL_FILE_CONTRACTID         = '@mozilla.org/file/local;1';
const PREFERENCES_CONTRACTID        = '@mozilla.org/preferences-service;1';
const IO_SERVICE_CONTRACTID         = '@mozilla.org/network/io-service;1';

function flickrPhotoService() { /*{{{*/
    dump ('CONSTRUCTOR: flickrPhotoService\n\n\n');
    this.init();
    this.state = flockIPhotoAPI.LOGGED_OUT;
}/*}}}*/

flickrPhotoService.prototype.init = function () { /*{{{*/
    this.api = new FlickrAPI();
    var inst = this;
    timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
    timerFunc = {
        inProgress: false,
        count: 0,
        notify: function(aTimer) {
            if(this.inProgress) return;
            this.inProgress = true;
            var oldToken = flock_getCharPref("flock.photo.flickr.token");
            if(oldToken && oldToken.length && !inst.api.isLoggedIn()) {
                inst.api.checkToken(this, oldToken);
            }
        },
        onResult: function(aXML) {
            this.inProgress = false;
        },
        onAuth: function(aXML) {
            this.inProgress = false;
            inst.state = flockIPhotoAPI.LOGGED_IN;
        },
        onError: function(aError) {
//            inst.state = flockIPhotoAPI.LOGIN_ERROR;
            inst.logout();
            this.inProgress = false;
        },
    }
    timerFunc.notify();
    timer.initWithCallback(timerFunc,5*1 * 1000,2);//re-check token
    //};
}/*}}}*/

flickrPhotoService.prototype.getAuthState = function () { /*{{{*/
    return this.state;
}/*}}}*/

flickrPhotoService.prototype.logout = function () { /*{{{*/
    this.api.logout();
    var cookieManager = Components.classes["@mozilla.org/cookiemanager;1"].
        getService(Components.interfaces.nsICookieManager);
    cookieManager.remove(".flickr.com","cookie_accid","/",false);
    cookieManager.remove(".flickr.com","cookie_epass","/",false);
    cookieManager.remove(".flickr.com","cookie_session","/",false);
    cookieManager.remove(".flickr.com","cookie_accid","/",false);
    cookieManager.remove(".flickr.com","use_master_until","/",false);
    this.state = flockIPhotoAPI.LOGGED_OUT;
}/*}}}*/

flickrPhotoService.prototype.login = function (aListener) { /*{{{*/
    var inst = this;
    var myListener = {
        onAuth: function() {
            inst.state = flockIPhotoAPI.LOGGED_IN;
            aListener.onAuth();
        },
        onError: function(aError) {
            inst.state = flockIPhotoAPI.LOGIN_ERROR;
            aListener.onError(aError);
        },
    }
    this.api.logout();
    inst.state = flockIPhotoAPI.LOGGING_IN;
    this.api.login(myListener);
}/*}}}*/

flickrPhotoService.prototype.isLoggedIn = function (aListener) { /*{{{*/
    return this.api.isLoggedIn();
//    if(this.api.isLoggedIn()) {
//        this.state = flockIPhotoAPI.LOGGED_IN;
//    }
}/*}}}*/

flickrPhotoService.prototype.getAuthUser = function () { /*{{{*/
    return this.api.getAuthUser();
}/*}}}*/

flickrPhotoService.prototype.isUploading = function () { /*{{{*/
    return this.running; 
}/*}}}*/

flickrPhotoService.prototype.dictionary2Params = function (aDictionary) { /*{{{*/
    var params = {};
    var obj = {};
    var count = {};
    var keys = aDictionary.getKeys(count, obj);
    for(var i=0;i<keys.length;++i) {
        var supports = aDictionary.getValue(keys[i]);
        var supportsString = supports.QueryInterface(Components.interfaces.nsISupportsString);
        var val = supportsString.toString();
        params[keys[i]] = val;
    }
    return params;
}/*}}}*/

flickrPhotoService.prototype.call = function (aListener, aMethod, aDictionary) { /*{{{*/
    var params = this.dictionary2Params(aDictionary);
    this.api.call(aListener, aMethod, params);
}/*}}}*/

flickrPhotoService.prototype.authenticatedCall = function (aListener, aMethod, aDictionary) { /*{{{*/
    var params = this.dictionary2Params(aDictionary);
    this.api.authenticatedCall(aListener, aMethod, params);
}/*}}}*/

flickrPhotoService.prototype.upload = function (aListener, aFile, aDictionary) { /*{{{*/
    var params = this.dictionary2Params(aDictionary);
    this.api.upload(aListener, aFile, params);
}/*}}}*/

flickrPhotoService.prototype.cancelUpload = function () { /*{{{*/
    try { this.api.uploader.req.abort(); }
    catch(e){};
}/*}}}*/

/* XXX: Horrible horrible hacks to get dojo to load */
//window = { location: { href: null, hash: null } };
//function XMLHttpRequest() { }

//loadLibraryFromSpec('chrome://browser/content/flock/contrib/dojo.js');
loadLibraryFromSpec('chrome://browser/content/flock/common/flocksafe.js');
loadLibraryFromSpec("chrome://browser/content/flock/contrib/md5.js");
loadLibraryFromSpec("chrome://browser/content/flock/photo/photoAPI.js");

function loadLibraryFromSpec(aSpec) {
    dump("loading " + aSpec + "\n");
    var loader = Components.classes['@mozilla.org/moz/jssubscript-loader;1']
        .getService(Components.interfaces.mozIJSSubScriptLoader);

    loader.loadSubScript(aSpec);
}

flickrPhotoService.prototype.flags = nsIClassInfo.Singleton;
flickrPhotoService.prototype.classDescription = "Flickr Photo Service";
flickrPhotoService.prototype.getInterfaces = function (count) {
    var interfaceList = [flockIPhotoAPI, nsIClassInfo];
    count.value = interfaceList.length;
    return interfaceList;
}
flickrPhotoService.prototype.getHelperForLanguage = function (count) {return null;}/*}}}*/

// the nsISupports implementation/*{{{*/
flickrPhotoService.prototype.QueryInterface =
function (iid) {
    if (!iid.equals(flockIPhotoAPI) && 
        !iid.equals(nsIClassInfo) &&
        !iid.equals(nsISupports))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    if (iid.equals(nsIRDFDataSource) && !this.dataSourceSetup) {
    }
    return this;
}/*}}}*/

// Module implementation/*{{{*/
var PhotoModule = new Object();

PhotoModule.registerSelf =
function (compMgr, fileSpec, location, type)
{
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(FLICKR_PHOTO_CID, 
                                    "Flickr Photo JS Component",
                                    FLICKR_PHOTO_CONTRACTID, 
                                    fileSpec, 
                                    location,
                                    type);
}

PhotoModule.getClassObject =
function (compMgr, cid, iid) {
    if (!cid.equals(FLICKR_PHOTO_CID))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    return PhotoServiceFactory;
}

PhotoModule.canUnload =
function(compMgr)
{
    debug("Unloading component.");
    return true;
}/*}}}*/
    
/* factory object *//*{{{*/
var PhotoServiceFactory = new Object();

PhotoServiceFactory.createInstance =
function (outer, iid) {
    debug("CI: " + iid);
    if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

    return (new flickrPhotoService()).QueryInterface(iid);
}/*}}}*/

/* entrypoint *//*{{{*/
function NSGetModule(compMgr, fileSpec) {
    return PhotoModule;
}/*}}}*/

FlickrAPI.prototype = new Object();

function FlickrAPI() { 
    var inst = this;
    this.api_key = "92c2a562f0e9c2ed8dfe78f42a7734c7"; 
    this.api_secret = "17b26c20558cf979"; 
    this.endpoint = "http://www.flickr.com/services/rest/"; 
    this.upload_endpoint = "http://www.flickr.com/services/upload/"; 
    this.auth_endpoint = "http://www.flickr.com/services/auth/"; 
    this.req = null;
    this.frob = null;
    this.lastToken = new Date();
    this.authUser = null;
    this.filesToDelete = new Array();
    this.setEndpoint = function(aEndpoint) {
        this.endpoint = aEndpoint;
    };

    this.getAuthUser = function() {
        return this.authUser;
    };
    this.getStoredToken = function(aListener) {
    };
    this.checkToken = function(aListener, aToken) {
        var tokenListener = {
            onResult: function (aXML) {
                var flickrUser = new FlickrUser();
                flickrUser.token = aXML.getElementsByTagName("token")[0].firstChild.nodeValue;
                var user = aXML.getElementsByTagName("user")[0];
                flickrUser.nsid = user.getAttribute("nsid");
                flickrUser.id = user.getAttribute("nsid");
                flickrUser.username = user.getAttribute("username");
                flickrUser.fullname = user.getAttribute("fullname");
                inst.authUser = flickrUser;

                dump("got token\n");
                inst.lastToken = new Date();
                aListener.onAuth();
            },
            onError: function (aError) {
                dump("error getting token\n");
                inst.frob = null;
                aListener.onError(null);
            },
        }
        var token = aToken;
        var params = {
            auth_token: token,
        };
        this.call(tokenListener, "flickr.auth.checkToken", params);
    };
    this.getToken = function(aListener) {
        var tokenListener = {
            onResult: function (aXML) {
                var flickrUser = new FlickrUser();
                flickrUser.token = aXML.getElementsByTagName("token")[0].firstChild.nodeValue;
                var user = aXML.getElementsByTagName("user")[0];
                flickrUser.nsid = user.getAttribute("nsid");
                flickrUser.id = user.getAttribute("nsid");
                flickrUser.username = user.getAttribute("username");
                flickrUser.fullname = user.getAttribute("fullname");
                inst.authUser = flickrUser;

                flock_setCharPref("flock.photo.flickr.token", flickrUser.token);
                dump("got token\n");
                inst.lastToken = new Date();
                aListener.onAuth();
            },
            onError: function (aError) {
                dump("error getting token\n");
                inst.frob = null;
                aListener.onError(null);
            },
        }
        var frob = this.frob;
        var params = {
            frob : frob,
        };
        this.call(tokenListener, "flickr.auth.getToken", params);
    };
    this.isLoggedIn = function() {
        if(this.authUser) return true;
        return false;
    };
    this.logout = function() {
        this.authUser = null;
        this.frob = null;
        this.token = null;
        flock_setCharPref("flock.photo.flickr.token", "");
    };
    this.login = function(aListener) {
        this.authUser = null;
        this.frob = null;
        var api = this;
        var frobListener = {
            onResult: function (aXML) {
                try {
                    var frobNode = aXML.getElementsByTagName("frob")[0];
                    var frob = frobNode.firstChild.nodeValue;
                    var authUrl = api.getAuthUrl(frob, "write");
                    api.frob = frob;
                        var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
                        var windowManagerInterface = windowManager.QueryInterface( Components.interfaces.nsIWindowMediator);
                        var topWindow = windowManagerInterface.getMostRecentWindow(null);
                        topWindow.open(authUrl,"_blank", "chrome,close,titlebar,resizable=yes,toolbar,dialog=no,scrollbars=yes,modal,centerscreen");
                        api.getToken(aListener);
                }
                catch(e) {
                }
            },
            onError: function (aXML) {
                aListener.onError(null);
            },
        }
        this.call(frobListener, "flickr.auth.getFrob", null);
    };
    this.upload = function(aListener, aPhoto, aParams) {
        this.uploader = new PhotoUploader();
        var myListener = {
            onResult: function(aXML) {
                var rsp = aXML.getElementsByTagName("rsp")[0];
                var stat = rsp.getAttribute("stat");
                if(stat!="ok") {
                    var err = aXML.getElementsByTagName("err")[0];
                    aListener.onError(unescape(err.getAttribute("msg")));
                }
                else {
                    aListener.onResult(aXML);
                }
            },
            onError: function(aXML) {
                aListener.onError(aXML);
            },
        }
        this.convertBools(aParams);
        this.convertTags(aParams);

        aParams.auth_token = this.authUser.token;
        aParams.api_key = this.api_key;
        aParams = this.appendSignature(aParams);

        this.uploader.setEndpoint(this.upload_endpoint);
        this.uploader.upload(myListener, aPhoto, aParams);
        return;
    };
    this.convertBools = function(aParams) {
        for(var p in aParams) {
            if(!p.match(/^is/)) continue;
            // I hope that this doesn't break anything
            if(aParams[p]=="true") aParams[p] = "1";
            if(aParams[p]=="false") aParams[p] = "0";
        }
    };
    this.convertTags = function(aParams) {
        for(var p in aParams) {
            if(p != "tags") continue;
            var tags = aParams[p].split(",");
            for(var i=0;i<tags.length;++i) {
                tags[i] = '"' + tags[i] + '"';
                tags[i] = tags[i].replace(/\"+/g,'"');
            }
            aParams[p] = tags.join(",");
        }
    };
    this.authenticatedCall = function(aListener, aMethod, aParams) {
        if(!aParams) aParams = {};
        aParams.auth_token = this.authUser.token;
        this.call(aListener, aMethod, aParams);
        var paramString = this.getParamString(aParams, true);
    };
    this.call = function(aListener, aMethod, aParams) {
        this.convertBools(aParams);
        this.convertTags(aParams);
        if(aParams==null) aParams = new Array();
        if(aMethod) aParams.method = aMethod;
        aParams.api_key = this.api_key;
        var paramString = this.getParamString(aParams, (aMethod==null));
        var url = this.endpoint + "?" + paramString;
        this._doCall(aListener, url, null);
    };
    this._doCall = function(aListener, aUrl, aContent) {
        dump("Sending " + aUrl + "\n");
        this.req = Components.classes[XMLHTTPREQUEST_CONTRACTID].createInstance(nsIXMLHttpRequest);
        this.req.open('POST', aUrl, true);
        var req = this.req;
        this.req.onreadystatechange = function (aEvt) {
            if(req.readyState == 4) {
                try {
                    if(req.status/100 == 2) {
                        var rsp = req.responseXML.getElementsByTagName("rsp")[0];
                        var stat = rsp.getAttribute("stat");
                        dump("\n" + req.responseText + "\n");
                        if(stat!="ok") {
                            var err = req.responseXML.getElementsByTagName("err")[0];
                            aListener.onError(unescape(err.getAttribute("msg")));
                        }
                        else {
                            aListener.onResult(req.responseXML);
                        }
                    }
                    else {
                        aListener.onError(null);
                    }
                } catch(e) {
                    aListener.onError(null);
                }
            }
        }
        this.req.send(null);
    };
    this.getAuthUrl = function(aFrob, aPerms) {
        var api_key = this.api_key;
        var params = {
            api_key: api_key,
            perms: aPerms,
            frob: aFrob,
        }

        var paramString = this.getParamString(params, true);
        return this.auth_endpoint + "?" + paramString;
    };
    this.getParamString = function(aParams, aNoMethod) {
        aParams = this.appendSignature(aParams);
        var rval = "";
        if(!aNoMethod) rval += "method=" + aParams.method + "&";

        var count = 0;
        for(var p in aParams) {
            if(p.match(/method/)) continue;

            if(count++!=0) rval += "&";
            rval += p + "=" + aParams[p];
        }
        return rval;
    };
    this.appendSignature = function(aParams) {
        var keys = new Array();
        for(var p in aParams) {
            keys.push(p);
        }
        keys.sort();
        var preHash = this.api_secret;
        for(var i=0;i<keys.length;++i) {
            preHash += keys[i] + aParams[keys[i]];
        }
        dump("preHash " + preHash + "\n");
        var sig = hex_md5(preHash);
        aParams.api_sig = sig;
        return aParams;
    };
}

function FlickrUser() { 
    this.token = "";
    this.perms = "";
    this.nsid = "";
    this.id = "";
    this.username = "";
    this.fullname = "";
}

FlickrUser.prototype = new Object();
