//
// BEGIN FLOCK GPL
//
// Copyright Flock Inc. 2005
// http://flock.com
//
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// END FLOCK GPL
//

const DEBUG = false;                // set to false to suppress debug messages
const FLOCK_PHOTO_CID               = Components.ID('{4799caea-6920-4549-b348-1992658581fe}');
const FLOCK_PHOTO_FNAME             = "flock_photo.rdf";

const nsISupports                   = Components.interfaces.nsISupports;
const nsIClassInfo                  = Components.interfaces.nsIClassInfo;
const nsIFactory                    = Components.interfaces.nsIFactory;
const nsIProperties                 = Components.interfaces.nsIProperties;
const nsILocalFile                  = Components.interfaces.nsILocalFile;
const nsIFile                       = Components.interfaces.nsIFile;
const nsIIOService                  = Components.interfaces.nsIIOService;
const nsIFileProtocolHandler        = Components.interfaces.nsIFileProtocolHandler;
const nsIRDFRemoteDataSource        = Components.interfaces.nsIRDFRemoteDataSource;
const nsIRDFDataSource              = Components.interfaces.nsIRDFDataSource;
const flockIPhotoService            = Components.interfaces.flockIPhotoService;
const flockIPhotoAPI                = Components.interfaces.flockIPhotoAPI;

const FLOCK_PHOTO_CONTRACTID        = '@mozilla.org/rdf/datasource;1?name=flock-photo';
const DIRECTORY_SERVICE_CONTRACTID  = '@mozilla.org/file/directory_service;1';
const LOCAL_FILE_CONTRACTID         = '@mozilla.org/file/local;1';
const PREFERENCES_CONTRACTID        = '@mozilla.org/preferences-service;1';
const IO_SERVICE_CONTRACTID         = '@mozilla.org/network/io-service;1';

const FLOCK_NS                      = 'http://flock.com/rdf#';
const FLOCK_RDF_UPLOAD_ROOT         = 'urn:flock:photo:upload';

function flockPhotoService() { /*{{{*/
    dump ('CONSTRUCTOR: flockPhotoService\n\n\n');
    var profd = Components.classes[DIRECTORY_SERVICE_CONTRACTID].
        getService(nsIProperties).get('ProfD', nsIFile);
    var file = Components.classes[LOCAL_FILE_CONTRACTID]
        .createInstance(nsILocalFile);
    try {
        file.initWithPath(profd.path);
    }
    catch(e) {
        var tmpd = Components.classes[DIRECTORY_SERVICE_CONTRACTID].
            getService(nsIProperties).get('TmpD', nsIFile);
        file.initWithPath(tmpd.path);
    }

    file.append(FLOCK_PHOTO_FNAME);
    if(!file.exists()) file.createUnique(0,0700);

    this.ds = new RDFDataSource('file://' + file.path);
    this.dataSource = this.ds.getRawDataSource()
        .QueryInterface (nsIRDFRemoteDataSource);
    this.uploadRoot = this.ds.getNode(FLOCK_RDF_UPLOAD_ROOT);
    this.uploadRoot.makeSeq();
    var cid = flock_getCharPref(FLOCK_PREF_PHOTO_API_CID);
    if(!cid) {
        dump("no photo api preference set.  Using flickr as default\n");
        cid = '@flock.com/?photo-api-flickr;1';
    }
    this.api = Components.classes[cid].getService(flockIPhotoAPI);
    //this.api = new (flock_photoGetAPIConstructor());
    this.api.init();
    this.init();
    this.listeners = new Array();
    this.running = false;
}/*}}}*/

flockPhotoService.prototype.authNotify = function() {
    for(var i=0;i<this.listeners.length;++i) {
        this.listeners[i].onAuthStateChange(this.api.getAuthState());
    }
}

flockPhotoService.prototype.init = function(aListener) {
        timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
        var inst = this;
        timerFunc = {
            loggedIn: false,
            notify: function(aTimer) {
                if(!this.isLoggedIn) {
                    if(inst.api.isLoggedIn()) {
                        dump("now we are logged in\n");
                        this.isLoggedIn = true;
                        inst.authNotify();
/*
                        for(var i=0;i<inst.listeners.length;++i) {
                            inst.listeners[i].onAuth();
                        }
*/
                    }
                }
                else {
                    if(!inst.api.isLoggedIn()) {
                        this.isLoggedIn = false;
                        inst.authNotify();
                        /*
                        for(var i=0;i<inst.listeners.length;++i) {
                            inst.listeners[i].onAuthLogout();
                        }
                        */
                    }
                }
            },
        }
        timer.initWithCallback(timerFunc,2*1 * 1000,2);//re-check token
}

flockPhotoService.prototype.addListener = function(aListener) {
    this.listeners.push(aListener);
    dump("adding listener " + this.listeners.length + "\n");
}

flockPhotoService.prototype.removeListener = function(aListener) {
    for(var i=0;i<this.listeners.length;++i) {
        if(aListener==this.listeners[i]) {
            this.listeners.splice(i,1);
            break;
        }
    }
}

flockPhotoService.prototype.logout = function (aListener) { /*{{{*/
    this.api.logout();
    this.authNotify();
    /*
    for(var i=0;i<this.listeners.length;++i) {
        this.listeners[i].onAuthLogout();
    }
    */
}/*}}}*/

flockPhotoService.prototype.login = function (aListener) { /*{{{*/
    var inst = this;
    var myListener = {
        onAuth: function() {
            aListener.onAuth();
            inst.authNotify();
            /*
            for(var i=0;i<inst.listeners.length;++i) {
                inst.listeners[i].onAuth();
            }
            */
        },
        onError: function(aError) {
            try {
                aListener.onError(aError);
                inst.authNotify();
                /*
                for(var i=0;i<inst.listeners.length;++i) {
                    inst.listeners[i].onAuthError(aError);
                }
                */
            }
            catch(e) {
                dump(e + " errrrrorororrorororor\n");
            }   
        },
    }
/*
    for(var i=0;i<this.listeners.length;++i) {
        this.listeners[i].onAuthStart();
    }
*/
    this.api.login(myListener);
    this.authNotify();//should be in LOGGING_IN state
}/*}}}*/

flockPhotoService.prototype.isLoggedIn = function (aListener) { /*{{{*/
    return this.api.isLoggedIn();
}/*}}}*/

flockPhotoService.prototype.getAuthUser = function () { /*{{{*/
    return this.api.getAuthUser();
}/*}}}*/

flockPhotoService.prototype.save = function (aListener) { /*{{{*/
    this.Flush();
}/*}}}*/

flockPhotoService.prototype.isUploading = function () { /*{{{*/
    return this.running; 
}/*}}}*/

flockPhotoService.prototype.getURI = function (aURI) { /*{{{*/
    return aURI; 
}/*}}}*/

flockPhotoService.prototype.dictionary2Params = function (aDictionary) { /*{{{*/
    var params = {};
    var obj = {};
    var count = {};
    var keys = aDictionary.getKeys(count, obj);
    for(var i=0;i<keys.length;++i) {
        var supports = aDictionary.getValue(keys[i]);
        var supportsString = supports.QueryInterface(Components.interfaces.nsISupportsString);
        var val = supportsString.toString();
        params[keys[i]] = val;
    }
    return params;
}/*}}}*/

flockPhotoService.prototype.call = function (aListener, aMethod, aDictionary) { /*{{{*/
//    var params = this.dictionary2Params(aDictionary);
    this.api.call(aListener, aMethod, aDictionary);
}/*}}}*/

flockPhotoService.prototype.authenticatedCall = function (aListener, aMethod, aDictionary) { /*{{{*/
//    var params = this.dictionary2Params(aDictionary);
    this.api.authenticatedCall(aListener, aMethod, aDictionary);
}/*}}}*/

flockPhotoService.prototype.cancelUpload = function () { /*{{{*/
    this.uploadListener.stop();
}/*}}}*/

flockPhotoService.prototype.upload = function () { /*{{{*/
    if(this.isUploading()) { return; }

    this.uploadListener = {
        init: function(aAPI, aKids, aSvc, aMax) {
            this.api = aAPI;
            this.svc = aSvc;
            this.kids = aKids;
            this.count = 0;
            this.max = aMax;
            this.imageUtils = flock_getImageUtils();
            this.svc.running = true;
            this.doUpload();
        },
        stop: function() {
            this.svc.running = false;
            this.api.cancelUpload();
            for(var i=0;i<this.svc.listeners.length;++i) {
                this.svc.listeners[i].onUploadComplete();
            }
        },
        doUpload: function() {
            if(!this.svc.running || !this.kids.hasMoreElements()) {
                //this.svc.running = false;
                if(this.svc.running) this.stop();
                return;
            }
            ++this.count;
            this.uploadResource = this.kids.getNext();
            var upload = this.svc.getPhotoUpload(this.uploadResource.getTarget(FLOCK_NS + "id").getValue());
            var url = upload.originalFileSpec;
            var rot = parseInt(upload.rotation);
            if(!isNaN(rot)) {
                if(rot%360!=0) {
                    var path = upload.originalFilePath;
                    var newName = flock.photo.util.getUniqueFname(path);
                    var newFile = flock_getTmpFile(newName);
                    flock.photo.util.rotate(rot, path, newFile.path);
                    url = flock_getURLFromFile(newFile);
                }
            }
            if(flock_getBoolPref(FLOCK_PREF_PHOTO_RESIZE)==true) {
                var maxDim = flock_getIntPref(FLOCK_PREF_PHOTO_RESIZE_DIMENSIONS);
                var path = upload.originalFilePath;
                /*
                var w = {}; var h = {};
                this.imageUtils.getDimensions(path, w, h);
                var newWidth;
                var newHeight;
                if(w.value>h.value) {
                    newWidth=maxDim;
                    newHeight=(h.value/w.value) * maxDim;
                }
                else {
                    newHeight=maxDim;
                    newWidth=(w.value/h.value) * maxDim;
                }
                */
                var newName = flock.photo.util.getUniqueFname(path);
                var newFile = flock_getTmpFile(newName);
                flock.photo.util.resize(maxDim, path, newFile.path);
                //this.imageUtils.resize(path, newFile.path, newWidth, newHeight);
                url = flock_getURLFromFile(newFile);
            }
            /*
            {
                var path = upload.originalFilePath;
                var newName = flock.photo.util.getUniqueFname(path);
                var newFile = flock_getTmpFile(newName);

                flock.photo.util.exify(path, newFile.path);

                url = flock_getURLFromFile(newFile);
            }
            */
            var params = new Array();
            params.title = upload.title;
            params.description = upload.description;
            params.is_family = upload.is_family;
            params.is_friend = upload.is_friend;
            params.is_public = upload.is_public;
            params.async = "1";
//            var tagArray = upload.tags.split(",");
//            params.tags = tagArray.join(" ");
            params.tags = upload.tags;
            this.currentUpload = upload;
            this.uploadResource.addTargetOnce(FLOCK_NS + "state", "uploading");
            for(var i=0;i<this.svc.listeners.length;++i) {
                this.svc.listeners[i].onUploadStart(this.currentUpload, this.count, this.max);
            }
            //this.api.upload(this, url, params);
            this.api.upload(this, url, params2Dictionary(params));
        },
        onResult: function() {
//            if(!this.kids.hasMoreElements()) {
                //this.svc.running = false;
//                this.stop();
//            }
            for(var i=0;i<this.svc.listeners.length;++i) {
                this.svc.listeners[i].onUpload(this.currentUpload, this.count, this.max);
            }
            this.svc.removePhotoUpload(this.currentUpload, false);
            this.doUpload();
        },
        onError: function(aError) {
            this.uploadResource.addTargetOnce(FLOCK_NS + "state", "error");
            for(var i=0;i<this.svc.listeners.length;++i) {
                if(this.svc.running)//don't send back error on cancelUpload
                    this.svc.listeners[i].onUploadError(this.currentUpload, this.count, this.max, null);
            }
            this.stop();
            dump("error uploading\n");
        },
    };
    var kids = this.uploadRoot.getChildren();
    this.uploadListener.init(this.api, kids, this, this.uploadRoot.getChildCount());
}/*}}}*/

flockPhotoService.prototype.savePhotoUpload = function (aUpload) { /*{{{*/
    debug("flockPhotoService.prototype.savePhotoUpload " + aUpload.id + "\n");
    var upload = this.ds.getNode(this.getURI(aUpload.id));
    upload.addTargetOnce(FLOCK_NS + "id", aUpload.id);
    upload.addTargetOnce(FLOCK_NS + "originalFileSpec", aUpload.originalFileSpec);
    upload.addTargetOnce(FLOCK_NS + "originalFilePath", aUpload.originalFilePath);
    upload.addTargetOnce(FLOCK_NS + "thumbFilePath", aUpload.thumbFilePath);
    upload.addTargetOnce(FLOCK_NS + "thumbFileSpec", aUpload.thumbFileSpec);
    upload.addTargetOnce(FLOCK_NS + "previewFilePath", aUpload.previewFilePath);
    upload.addTargetOnce(FLOCK_NS + "previewFileSpec", aUpload.previewFileSpec);
    upload.addTargetOnce(FLOCK_NS + "title", aUpload.title);
    upload.addTargetOnce(FLOCK_NS + "is_public", aUpload.is_public);
    upload.addTargetOnce(FLOCK_NS + "is_friend", aUpload.is_friend);
    upload.addTargetOnce(FLOCK_NS + "is_family", aUpload.is_family);
    upload.addTargetOnce(FLOCK_NS + "tags", aUpload.tags);
    upload.addTargetOnce(FLOCK_NS + "state", aUpload.state);
    upload.addTargetOnce(FLOCK_NS + "description", aUpload.description);
    upload.addTargetOnce(FLOCK_NS + "rotation", aUpload.rotation);
    this.Flush();
}/*}}}*/

flockPhotoService.prototype.addPhotoUpload = function (aReference, aUpload) { /*{{{*/
    debug("flockPhotoService.prototype.addPhotoUpload " + aUpload.state + "\n");
    this.removePhotoUpload(aUpload, true);

    var newUpload = this.ds.getNode(this.getURI(aUpload.id));
    this.savePhotoUpload(aUpload);
    if(aReference) {
        var ref = this.ds.getNode(this.getURI(aReference.id));
        var index = this.uploadRoot.getChildIndex(ref);
        dump("index " + index + "\n")
        this.uploadRoot.addChildAt(newUpload, index);
    }
    else {
        this.uploadRoot.addChild(newUpload);
    }
    this.Flush();
    for(var i=0;i<this.listeners.length;++i) {
        this.listeners[i].onUploadAdd(newUpload);
    }
}/*}}}*/

flockPhotoService.prototype.rotatePhotoUpload = function (aUpload, aDegrees) { /*{{{*/
}/*}}}*/

flockPhotoService.prototype.removePhotoUpload = function (aUpload, aDontDelete) { /*{{{*/
    try {
        aUpload = this.getPhotoUpload(aUpload.id);
    }
    catch(e) {}
    var toDelete = this.ds.getNode(this.getURI(aUpload.id));
    this.uploadRoot.removeChild(toDelete);
    this.ds.deleteRecursive(toDelete);
    this.Flush();
    if(!aDontDelete) {
        try {
        flock.photo.util.removeFileBySpec(aUpload.originalFileSpec);
        flock.photo.util.removeFileBySpec(aUpload.previewFileSpec);
        flock.photo.util.removeFileBySpec(aUpload.thumbFileSpec);
        flock.photo.util.removeFileByPath(aUpload.originalFilePath);
        flock.photo.util.removeFileByPath(aUpload.previewFilePath);
        flock.photo.util.removeFileByPath(aUpload.thumbFilePath);
        } catch(e) {}
    }
    for(var i=0;i<this.listeners.length;++i) {
        this.listeners[i].onUploadRemove();
    }
}/*}}}*/

flockPhotoService.prototype.reorderPhotoUpload = function (aReference, aUpload) { /*{{{*/
    this.addPhotoUpload();
    return;
/*
    //I wish the code below worked :( no time to investigate right now
    //this.removePhotoUpload(aUpload);
    var upload = this.ds.getNode(this.getURI(aUpload.id));
    this.uploadRoot.removeChild(upload);
    var ref = this.ds.getNode(this.getURI(aReference.id));
    if(aReference) {
        var index = this.uploadRoot.getChildIndex(ref);
        this.uploadRoot.addChildAt(upload, index);
    }
    else {
        this.uploadRoot.addChild(upload);
    }
    this.Flush();
*/
}/*}}}*/

flockPhotoService.prototype.getPhotoUpload = function (aURI) { /*{{{*/
    var upload = this.ds.getNode(this.getURI(aURI));
    var rval = new Object();
    rval.id = upload.getTarget(FLOCK_NS + "id").getValue();
    rval.originalFileSpec = upload.getTarget(FLOCK_NS + "originalFileSpec").getValue();
    rval.originalFilePath = upload.getTarget(FLOCK_NS + "originalFilePath").getValue();
    rval.thumbFileSpec = upload.getTarget(FLOCK_NS + "thumbFileSpec").getValue();
    rval.thumbFilePath = upload.getTarget(FLOCK_NS + "thumbFilePath").getValue();
    rval.previewFileSpec = upload.getTarget(FLOCK_NS + "previewFileSpec").getValue();
    rval.previewFilePath = upload.getTarget(FLOCK_NS + "previewFilePath").getValue();
    rval.title = upload.getTarget(FLOCK_NS + "title").getValue();
    rval.is_public = upload.getTarget(FLOCK_NS + "is_public").getValue();
    rval.is_friend = upload.getTarget(FLOCK_NS + "is_friend").getValue();
    rval.is_family = upload.getTarget(FLOCK_NS + "is_family").getValue();
    rval.tags = upload.getTarget(FLOCK_NS + "tags").getValue();
    rval.state = upload.getTarget(FLOCK_NS + "state").getValue();
    rval.description = upload.getTarget(FLOCK_NS + "description").getValue();
    rval.rotation = upload.getTarget(FLOCK_NS + "rotation").getValue();
    return rval;
}/*}}}*/

/* XXX: Horrible horrible hacks to get dojo to load */
//window = { location: { href: null, hash: null } };
//function XMLHttpRequest() { }

//loadLibraryFromSpec('chrome://browser/content/flock/contrib/dojo.js');
loadLibraryFromSpec('chrome://browser/content/flock/common/flocksafe.js');
loadLibraryFromSpec('chrome://browser/content/flock/contrib/md5.js');
loadLibraryFromSpec('chrome://browser/content/flock/contrib/rdfds.js');
loadLibraryFromSpec('chrome://browser/content/flock/photo/photoAPI.js');
loadLibraryFromSpec('chrome://browser/content/flock/photo/photo.js');

function loadLibraryFromSpec(aSpec) {
    dump("loading " + aSpec + "\n");
    var loader = Components.classes['@mozilla.org/moz/jssubscript-loader;1']
        .getService(Components.interfaces.mozIJSSubScriptLoader);

    loader.loadSubScript(aSpec);
}

flockPhotoService.prototype.URI = 'rdf:flock-photo';
flockPhotoService.prototype.GetSource = function (aProperty, aTarget, aTruthValue) { /*{{{*/
    return this.dataSource.GetSource (aProperty, aTarget, aTruthValue); 
}/*}}}*/
flockPhotoService.prototype.GetSources = function (aProperty, aTarget, aTruthValue) { /*{{{*/
    return this.dataSource.GetSources (aProperty, aTarget, aTruthValue); 
}/*}}}*/
flockPhotoService.prototype.GetTarget = function (aSource, aProperty, aTruthValue) { /*{{{*/
    return this.dataSource.GetTarget (aSource, aProperty, aTruthValue); 
}/*}}}*/
flockPhotoService.prototype.GetTargets = function (aSource, aProperty, aTruthValue) { /*{{{*/
    return this.dataSource.GetTargets (aSource, aProperty, aTruthValue); 
}/*}}}*/
flockPhotoService.prototype.Assert = function (aSource, aProperty, aTarget, aTruthValue) { /*{{{*/
    this.dataSource.Assert (aSource, aProperty, aTarget, aTruthValue); 
}/*}}}*/
flockPhotoService.prototype.Unassert = function (aSource, aProperty, aTarget) { /*{{{*/
    this.dataSource.Unassert (aSource, aProperty, aTarget); 
}/*}}}*/
flockPhotoService.prototype.Change = function (aSource, aProperty, aOldTarget, aNewTarget) { /*{{{*/
    this.dataSource.Change (aSource, aProperty, aOldTarget, aNewTarget); 
}/*}}}*/
flockPhotoService.prototype.Move = function (aOldSource, aNewSource, aProperty, aTarget) { /*{{{*/
    this.dataSource.Move (aOldSource, aNewSource, aProperty, aTarget); 
}/*}}}*/
flockPhotoService.prototype.HasAssertion = function (aSource, aProperty, aTarget, aTruthValue) { /*{{{*/
    return this.dataSource.HasAssertion (aSource, aProperty, aTarget, aTruthValue); 
}/*}}}*/
flockPhotoService.prototype.AddObserver = function (aObserver) { /*{{{*/
    this.dataSource.AddObserver (aObserver); 
}/*}}}*/
flockPhotoService.prototype.RemoveObserver = function (aObserver) { /*{{{*/
    this.dataSource.RemoveObserver (aObserver); 
}/*}}}*/
flockPhotoService.prototype.ArcLabelsIn = function (aNode) { /*{{{*/
    return this.dataSource.ArcLabelsIn (aNode); 
}/*}}}*/
flockPhotoService.prototype.ArcLabelsOut = function (aNode) { /*{{{*/
    return this.dataSource.ArcLabelsOut (aNode); 
}/*}}}*/
flockPhotoService.prototype.GetAllResources = function () { /*{{{*/
    return this.dataSource.GetAllResources (); 
}/*}}}*/
flockPhotoService.prototype.IsCommandEnabled = function (aSources, aCommand, aArguments) { /*{{{*/
    return this.dataSource.IsCommandEnabled (aSources, aCommand, aArguments); 
}/*}}}*/
flockPhotoService.prototype.DoCommand = function (aSources, aCommand, aArguments) { /*{{{*/
    this.dataSource.DoCommand (aSources, aCommand, aArguments); 
}/*}}}*/
flockPhotoService.prototype.GetAllCmds = function (aSource) { /*{{{*/
    return this.dataSource.GetAllCmds (aSource); 
}/*}}}*/
flockPhotoService.prototype.hasArcIn = function (aNode, aArc) { /*{{{*/
    return this.dataSource.hasArcIn (aNode, aArc); 
}/*}}}*/
flockPhotoService.prototype.hasArcOut = function (aNode, aArc) { /*{{{*/
    return this.dataSource.hasArcOut (aNode, aArc); 
}/*}}}*/
flockPhotoService.prototype.beginUpdateBatch = function () { /*{{{*/
    this.dataSource.beginUpdateBatch (); 
}/*}}}*/
flockPhotoService.prototype.endUpdateBatch = function () { /*{{{*/
    this.dataSource.endUpdateBatch (); 
}/*}}}*/
flockPhotoService.prototype.Flush = function () { /*{{{*/
    this.dataSource.Flush (); 
}/*}}}*/
flockPhotoService.prototype.FlushTo = function (aURI) { /*{{{*/
    this.dataSource.FlushTo (aURI); 
}/*}}}*/
flockPhotoService.prototype.Init = function (aURI) { /*{{{*/
    this.dataSource.Init (aURI); 
}/*}}}*/
flockPhotoService.prototype.Refresh = function (aBlocking) { /*{{{*/
    this.dataSource.Refresh (aBlocking); 
}/*}}}*/

flockPhotoService.prototype.flags = nsIClassInfo.Singleton;
flockPhotoService.prototype.classDescription = "Flock Photo Service";
flockPhotoService.prototype.getInterfaces = function (count) {
    var interfaceList = [flockIPhotoService, nsIClassInfo];
    count.value = interfaceList.length;
    return interfaceList;
}
flockPhotoService.prototype.getHelperForLanguage = function (count) {return null;}/*}}}*/

// the nsISupports implementation/*{{{*/
flockPhotoService.prototype.QueryInterface =
function (iid) {
    if (!iid.equals(flockIPhotoService) && 
        !iid.equals(nsIRDFDataSource) &&
        !iid.equals(nsIClassInfo) &&
        !iid.equals(nsISupports))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    if (iid.equals(nsIRDFDataSource) && !this.dataSourceSetup) {
    }
    return this;
}/*}}}*/

// Module implementation/*{{{*/
var PhotoModule = new Object();

PhotoModule.registerSelf =
function (compMgr, fileSpec, location, type)
{
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);

    compMgr.registerFactoryLocation(FLOCK_PHOTO_CID, 
                                    "Flock Photo JS Component",
                                    FLOCK_PHOTO_CONTRACTID, 
                                    fileSpec, 
                                    location,
                                    type);
}

PhotoModule.getClassObject =
function (compMgr, cid, iid) {
    if (!cid.equals(FLOCK_PHOTO_CID))
        throw Components.results.NS_ERROR_NO_INTERFACE;
    
    if (!iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    
    return PhotoServiceFactory;
}

PhotoModule.canUnload =
function(compMgr)
{
    debug("Unloading component.");
    //gPhotoService.dataSource.Flush();
    return true;
}/*}}}*/
    
/* factory object *//*{{{*/
var PhotoServiceFactory = new Object();

PhotoServiceFactory.createInstance =
function (outer, iid) {
    debug("CI: " + iid);
    if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

    return (new flockPhotoService()).QueryInterface(iid);
}/*}}}*/

/* entrypoint *//*{{{*/
function NSGetModule(compMgr, fileSpec) {
    return PhotoModule;
}/*}}}*/
/*
function array2params(aArray) {
    var rval = {};
    for(var i=0;i<aArray.length/2;i+=2) {
        rval[aArray[i]] = aArray[i+1]; 
    }
    return rval;
}

flockPhotoService.prototype.photoCall(aListener, aMethod, aParams, aParamsLen, aCount, aPage) {
    var method = "call_" + aMethod;
    var call = this.api[method];

    if(!call) throw new Error("Unsupported method " + method);
    var myListener = {
        onResult: function(aResult) {
            dump("HAVE SOME RESULTS!!\n");
            return;
        },
        onError: function(aError) {
        },
    }

    var params = array2params(aParams);

    try {
        call(myListener, params, aCount, aPage);
    }
    catch(e) {
        throw new Error("Unsupported method " + method);
    }
}
*/

